@@ -81,7 +81,11 @@ struct {
 	{ 0, "Username required"		},	/* 35: UPSCLI_ERR_USERREQUIRED */
 	{ 0, "SSL is not available",		},	/* 36: UPSCLI_ERR_SSLFAIL */
 	{ 2, "SSL error: %s",			},	/* 37: UPSCLI_ERR_SSLERR */
-
+	{ 0, "Server disconnected",		},	/* 38: UPSCLI_ERR_SRVDISC */
+	{ 0, "Driver not connected",		},	/* 39: UPSCLI_ERR_DRVNOTCONN */
+	{ 0, "Memory allocation failure",	},	/* 40: UPSCLI_ERR_NOMEM */
+	{ 3, "Parse error: %s",			},	/* 41: UPSCLI_ERR_PARSE */
+	{ 0, "Protocol error",			},	/* 42: UPSCLI_ERR_PROTOCOL */
 };
 
 /* make sure we're using a struct that's been through upscli_connect */
@@ -141,6 +145,12 @@ const char *upscli_strerror(UPSCONN *ups)
 				"SSL error, but SSL wasn't enabled at compile-time");
 #endif	/* HAVE_SSL */
 			return ups->errbuf;
+
+		case 3:		/* parsing (parseconf) error */
+			snprintf(ups->errbuf, UPSCLI_ERRBUF_LEN,
+				upscli_errlist[ups->upserror].str,
+				ups->pc_ctx->errmsg);
+			return ups->errbuf;
 	}
 
 	/* fallthrough */
@@ -160,9 +170,9 @@ static void upscli_closefd(UPSCONN *ups)
 		ups->ssl = NULL;
 	}
 
-	if (ups->ctx) {
-		SSL_CTX_free(ups->ctx);
-		ups->ctx = NULL;
+	if (ups->ssl_ctx) {
+		SSL_CTX_free(ups->ssl_ctx);
+		ups->ssl_ctx = NULL;
 	}
 #endif
 
@@ -294,7 +304,7 @@ int upscli_sslcert(UPSCONN *ups, const char *dir, const char *file, int verify)
 int upscli_sslinit(UPSCONN *ups)
 {
 	int	ret;
-	char	buf[SMALLBUF];
+	char	buf[UPSCLI_NETBUF_LEN];
 
 	if (!ups)
 		return -1;
@@ -317,12 +327,26 @@ int upscli_sslinit(UPSCONN *ups)
 		return -1;
 	}
 
-	ret = upscli_read(ups, buf, sizeof(buf));
+	memset(buf, '\0', sizeof(buf));
+	ret = read(ups->fd, buf, sizeof(buf));
 
-	if (ret < 0)
+	if (ret == 0) {			/* server hung up on us! */
+		ups->upserror = UPSCLI_ERR_SRVDISC;
+		ups->syserrno = 0;
+
+		upscli_closefd(ups);
 		return -1;
+	}
 
-	if (strcmp(buf, "OK STARTTLS") != 0)
+	if (ret < 1) {
+		ups->upserror = UPSCLI_ERR_READ;
+		ups->syserrno = errno;
+
+		upscli_closefd(ups);
+		return -1;
+	}
+
+	if (strncmp(buf, "OK STARTTLS", 11) != 0)
 		return 0;		/* not supported */
 
 	/* upsd is happy, so let's crank up the client */
@@ -330,12 +354,12 @@ int upscli_sslinit(UPSCONN *ups)
 	SSL_library_init();
 	SSL_load_error_strings();
 
-	ups->ctx = SSL_CTX_new(TLSv1_client_method());
+	ups->ssl_ctx = SSL_CTX_new(TLSv1_client_method());
 
-	if (!ups->ctx)
+	if (!ups->ssl_ctx)
 		return 0;
 	
-	ups->ssl = SSL_new(ups->ctx);
+	ups->ssl = SSL_new(ups->ssl_ctx);
 
 	if (!ups->ssl)
 		return 0;
@@ -358,7 +382,7 @@ int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify)
 	if (!ups)
 		return -1;
 
-	if (!ups->ctx) {
+	if (!ups->ssl_ctx) {
 		ups->upserror = UPSCLI_ERR_INVALIDARG;
 		return -1;
 	}
@@ -372,7 +396,7 @@ int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify)
 			return -1;
 	}
 
-	ret = SSL_CTX_load_verify_locations(ups->ctx, file, path);
+	ret = SSL_CTX_load_verify_locations(ups->ssl_ctx, file, path);
 
 	if (ret != 1) {
 		ups->upserror = UPSCLI_ERR_SSLERR;
@@ -398,7 +422,17 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
 	ups->upserror = 0;
 	ups->syserrno = 0;
 	ups->upsclient_magic = UPSCLIENT_MAGIC;
-	ups->ctx = NULL;
+
+	ups->pc_ctx = malloc(sizeof(PCONF_CTX));
+
+	if (!ups->pc_ctx) {
+		ups->upserror = UPSCLI_ERR_NOMEM;
+		return -1;		
+	}
+
+	pconf_init(ups->pc_ctx, NULL);
+
+	ups->ssl_ctx = NULL;
 	ups->ssl = NULL;
 
 	if (!host) {
@@ -448,12 +482,27 @@ int upscli_connect(UPSCONN *ups, const char *host, int port, int flags)
 		return -1;
 	}
 
-	ups->host = xstrdup(host);
+	/* don't use xstrdup for cleaner linking (fewer dependencies) */
+	ups->host = strdup(host);
+
+	if (!ups->host) {
+		close(ups->fd);
+		ups->fd = -1;
+
+		ups->upserror = UPSCLI_ERR_NOMEM;
+		return -1;
+	}
+
 	ups->port = port;
 
-	if (flags & UPSCLI_CONN_TRYSSL)
+	if (flags & UPSCLI_CONN_TRYSSL) {
 		upscli_sslinit(ups);
 
+		/* see if something made us die inside sslinit */
+		if (ups->upserror != 0)
+			return -1;
+	}
+
 	if (flags & UPSCLI_CONN_REQSSL) {
 		if (upscli_sslinit(ups) != 1) {
 			ups->upserror = UPSCLI_ERR_SSLFAIL;
@@ -495,6 +544,7 @@ static struct {
 	{ UPSCLI_ERR_UNKCOMMAND,	"UNKNOWN-COMMAND"	},
 	{ UPSCLI_ERR_INVPASSWORD,	"INVALID-PASSWORD"	},
 	{ UPSCLI_ERR_USERREQUIRED,	"USERNAME-REQUIRED"	},
+	{ UPSCLI_ERR_DRVNOTCONN,	"DRIVER-NOT-CONNECTED"	},
 	
 	{ 0,			NULL,		}
 };
@@ -517,7 +567,8 @@ static int upscli_errcheck(UPSCONN *ups, char *buf)
 
 	/* look it up in the table */
 	for (i = 0; upsd_errlist[i].text != NULL; i++) {
-		if (!strcmp(&buf[4], upsd_errlist[i].text)) {
+		if (!strncmp(&buf[4], upsd_errlist[i].text,
+			strlen(upsd_errlist[i].text))) {
 			ups->upserror = upsd_errlist[i].errnum;
 			return -1;
 		}
@@ -528,11 +579,204 @@ static int upscli_errcheck(UPSCONN *ups, char *buf)
 	return -1;
 }
 
+static void build_cmd(char *buf, size_t bufsize, const char *cmdname,
+	int numarg, const char **arg)
+{
+	int	i;
+	size_t	len;
+	char	enc[UPSCLI_NETBUF_LEN], *format;
+
+	memset(buf, '\0', bufsize);
+	snprintf(buf, bufsize, "%s", cmdname);
+
+	/* encode all arguments so they arrive intact */
+	for (i = 0; i < numarg; i++) {
+
+		if (strchr(arg[i], ' '))
+			format = " \"%s\"";	/* wrap in "" */
+		else
+			format = " %s";
+
+		/* snprintfcat would tie us to common */
+
+		len = strlen(buf);
+		snprintf(buf + len, bufsize - len, format, 
+			pconf_encode(arg[i], enc, sizeof(enc)));
+	}
+
+	len = strlen(buf);
+	snprintf(buf + len, bufsize - len, "\n");
+}
+
+/* make sure upsd is giving us what we asked for */
+static int verify_resp(int num, const char **q, char **a)
+{
+	int	i;
+
+	for (i = 0; i < num; i++) {
+		if (strcasecmp(q[i], a[i]) != 0) {
+
+			/* FUTURE: handle -/+ options here */
+			return 0;	/* mismatch */
+		}
+	}
+
+	return 1;	/* OK */
+}
+
+int upscli_get(UPSCONN *ups, int numq, const char **query, 
+		int *numa, char ***answer)
+{
+	int	ret;
+	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
+	
+	if (numq < 1) {
+		ups->upserror = UPSCLI_ERR_INVALIDARG;
+		return -1;
+	}
+
+	/* create the string to send to upsd */
+	build_cmd(cmd, sizeof(cmd), "GET", numq, query);
+
+	ret = net_write(ups, cmd, strlen(cmd));
+
+	if (ret < 0)
+		return -1;
+
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+		return -1;
+
+	if (upscli_errcheck(ups, tmp) != 0)
+		return -1;
+
+	if (!pconf_line(ups->pc_ctx, tmp)) {
+		ups->upserror = UPSCLI_ERR_PARSE;
+		return -1;
+	}
+
+	/* q: [GET] VAR <ups> <var>   *
+	 * a: VAR <ups> <var> <val> */
+
+	if (ups->pc_ctx->numargs < numq) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	if (!verify_resp(numq, query, ups->pc_ctx->arglist)) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	*numa = ups->pc_ctx->numargs;
+	*answer = ups->pc_ctx->arglist;
+
+	return 0;
+}
+
+int upscli_list_start(UPSCONN *ups, int numq, const char **query)
+{
+	int	ret;
+	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN];
+
+	if (numq < 1) {
+		ups->upserror = UPSCLI_ERR_INVALIDARG;
+		return -1;
+	}
+
+	/* create the string to send to upsd */
+	build_cmd(cmd, sizeof(cmd), "LIST", numq, query);
+
+	ret = net_write(ups, cmd, strlen(cmd));
+
+	if (ret < 0)
+		return -1;
+
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+		return -1;
+
+	if (upscli_errcheck(ups, tmp) != 0)
+		return -1;
+
+	if (!pconf_line(ups->pc_ctx, tmp)) {
+		ups->upserror = UPSCLI_ERR_PARSE;
+		return -1;
+	}
+
+	if (ups->pc_ctx->numargs < 2) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	/* the response must start with BEGIN LIST */
+	if ((strcasecmp(ups->pc_ctx->arglist[0], "BEGIN") != 0) || 
+		(strcasecmp(ups->pc_ctx->arglist[1], "LIST") != 0)) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	/* q: [LIST] VAR <ups>       *
+	 * a: [BEGIN LIST] VAR <ups> */
+
+	/* compare q[0]... to a[2]... */
+
+	if (!verify_resp(numq, query, &ups->pc_ctx->arglist[2])) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	return 0;
+}
+
+int upscli_list_next(UPSCONN *ups, int numq, const char **query, 
+		int *numa, char ***answer)
+{
+	char	tmp[UPSCLI_NETBUF_LEN];
+
+	if (upscli_read(ups, tmp, sizeof(tmp)) != 0)
+		return -1;
+
+	if (upscli_errcheck(ups, tmp) != 0)
+		return -1;
+
+	if (!pconf_line(ups->pc_ctx, tmp)) {
+		ups->upserror = UPSCLI_ERR_PARSE;
+		return -1;
+	}
+
+	if (ups->pc_ctx->numargs < 1) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	*numa = ups->pc_ctx->numargs;
+	*answer = ups->pc_ctx->arglist;
+
+	/* see if this is the end */
+	if (ups->pc_ctx->numargs >= 2) {
+		if ((!strcmp(ups->pc_ctx->arglist[0], "END")) &&
+			(!strcmp(ups->pc_ctx->arglist[1], "LIST")))
+			return 0;
+	}
+
+	/* q: VAR <ups> */
+	/* a: VAR <ups> <val> */
+
+	if (!verify_resp(numq, query, ups->pc_ctx->arglist)) {
+		ups->upserror = UPSCLI_ERR_PROTOCOL;
+		return -1;
+	}
+
+	/* just another part of the list */
+	return 1;
+}
+
+/* --- */
+
 int upscli_getvar(UPSCONN *ups, const char *upsname, const char *var, 
 			char *buf, size_t buflen)
 {
-	int	ret;
-	char	cmd[SMALLBUF], tmp[LARGEBUF], *ptr;
+	int	ret, skiplen;
+	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN], *ptr;
 
 	if (!ups)
 		return -1;
@@ -572,11 +816,28 @@ int upscli_getvar(UPSCONN *ups, const char *upsname, const char *var,
 		return -1;
 	}
 
-	if (!upsname)		/* skip "ANS <var> " */
-		ptr = tmp + strlen(var) + 5;
-	else			/* skip "ANS <var>@<upsname> " */
-		ptr = tmp + strlen(var) + 5 + strlen(upsname) + 1;
+	if (!upsname) {
 
+		/* skip "ANS <var> " */
+		skiplen = 4 + strlen(var) + 1;
+
+	} else {
+
+		/* skip "ANS <var>@<upsname> " */
+		skiplen = 4 + strlen(var) + 1 + strlen(upsname) + 1;
+	}
+
+	/* make sure we got enough data */
+	if (strlen(tmp) <= skiplen) {
+		ups->upserror = UPSCLI_ERR_INVRESP;
+
+		/* try to copy some of it for later inspection */
+		snprintf(buf, buflen, "%s", tmp);
+
+		return -1;
+	}
+
+	ptr = tmp + skiplen;
 	snprintf(buf, buflen, "%s", ptr);
 
 	return 0;
@@ -598,7 +859,7 @@ int upscli_getlist(UPSCONN *ups, const char *upsname, int listtype,
 			char *buf, size_t buflen)
 {
 	int	ret, i, cmdpos;
-	char	cmd[SMALLBUF], tmp[LARGEBUF], *ptr;
+	char	cmd[UPSCLI_NETBUF_LEN], tmp[UPSCLI_NETBUF_LEN], *ptr;
 
 	if (!ups)
 		return -1;
@@ -725,6 +986,8 @@ int upscli_readline(UPSCONN *ups, char *buf, size_t buflen)
 	return 0;
 }
 
+/* FUTURE: this will not allow a missing upsname component */
+
 /* split [upsname@]hostname[:port] into separate components */
 int upscli_splitname(const char *buf, char **upsname, char **hostname,
 			int *port)
@@ -740,7 +1003,13 @@ int upscli_splitname(const char *buf, char **upsname, char **hostname,
 
 	if (ap) {
 		*ap++ = '\0';
-		*upsname = xstrdup(tmp);
+		*upsname = strdup(tmp);
+
+		if (!*upsname) {
+			fprintf(stderr, "upscli_splitname: strdup failed\n");
+			return -1;
+		}
+
 		ptr = ap;
 
 	} else {
@@ -753,14 +1022,26 @@ int upscli_splitname(const char *buf, char **upsname, char **hostname,
 
 	if (cp) {
 		*cp++ = '\0';
-		*hostname = xstrdup(ptr);
+		*hostname = strdup(ptr);
+
+		if (!*hostname) {
+			fprintf(stderr, "upscli_splitname: strdup failed\n");
+			return -1;
+		}
+
 		ptr = cp;
 
 		*port = strtol(ptr, (char **) NULL, 10);
 
 	} else {
 
-		*hostname = xstrdup(ptr);
+		*hostname = strdup(ptr);
+
+		if (!*hostname) {
+			fprintf(stderr, "upscli_splitname: strdup failed\n");
+			return -1;
+		}
+
 		*port = PORT;
 	}
 
@@ -783,6 +1064,12 @@ int upscli_disconnect(UPSCONN *ups)
 		upscli_closefd(ups);
 	}
 
+	if (ups->pc_ctx) {
+		pconf_finish(ups->pc_ctx);
+		free(ups->pc_ctx);
+		ups->pc_ctx = NULL;
+	}
+
 	if (ups->host)
 		free(ups->host);
 
@@ -792,3 +1079,38 @@ int upscli_disconnect(UPSCONN *ups)
 	return 0;
 }
 
+int upscli_fd(UPSCONN *ups)
+{
+	if (!ups)
+		return -1;
+
+	if (!upscli_checkmagic(ups))
+		return -1;
+
+	return ups->fd;
+}
+
+int upscli_upserror(UPSCONN *ups)
+{
+	if (!ups)
+		return -1;
+
+	if (!upscli_checkmagic(ups))
+		return -1;
+
+	return ups->upserror;
+}
+
+int upscli_ssl(UPSCONN *ups)
+{
+	if (!ups)
+		return -1;
+
+	if (!upscli_checkmagic(ups))
+		return -1;
+
+	if (ups->ssl)
+		return 1;
+
+	return 0;
+}
