@@ -26,6 +26,9 @@
 #endif
 
 #define UPSCLI_ERRBUF_LEN	256
+#define UPSCLI_NETBUF_LEN	512	/* network i/o buffer */
+
+#include "parseconf.h"
 
 typedef struct {
 	char	*host;
@@ -35,13 +38,16 @@ typedef struct {
 	int	upserror;
 	int	syserrno;
 	int	upsclient_magic;
+
+	PCONF_CTX	*pc_ctx;
+
 	char	errbuf[UPSCLI_ERRBUF_LEN];
 
 #ifdef HAVE_SSL
-	SSL_CTX	*ctx;
+	SSL_CTX	*ssl_ctx;
 	SSL	*ssl;
 #else
-	void	*ctx;
+	void	*ssl_ctx;
 	void	*ssl;
 #endif
 
@@ -51,12 +57,26 @@ const char *upscli_strerror(UPSCONN *ups);
 
 int upscli_connect(UPSCONN *ups, const char *host, int port, int flags);
 
+/* --- functions that only use the new names --- */
+
+int upscli_get(UPSCONN *ups, int numq, const char **query, 
+		int *numa, char ***answer);
+
+int upscli_list_start(UPSCONN *ups, int numq, const char **query);
+
+int upscli_list_next(UPSCONN *ups, int numq, const char **query,
+		int *numa, char ***answer);
+
+/* --- functions that only use the old names --- */
+
 int upscli_getvar(UPSCONN *ups, const char *upsname, const char *var, 
 			char *buf, size_t buflen);
 
 int upscli_getlist(UPSCONN *ups, const char *upsname, int listtype, 
 			char *buf, size_t buflen);
 
+/* --- */
+
 int upscli_sendline(UPSCONN *ups, const char *buf, size_t buflen);
 
 int upscli_readline(UPSCONN *ups, char *buf, size_t buflen);
@@ -68,6 +88,14 @@ int upscli_sslcert(UPSCONN *ups, const char *file, const char *path, int verify)
 
 int upscli_disconnect(UPSCONN *ups);
 
+/* these functions return elements from UPSCONN to avoid direct references */
+
+int upscli_fd(UPSCONN *ups);
+int upscli_upserror(UPSCONN *ups);
+
+/* returns 1 if SSL mode is active for this connection */
+int upscli_ssl(UPSCONN *ups);	
+
 /* upsclient error list */
 
 #define UPSCLI_ERR_UNKNOWN	0	/* Unknown error */
@@ -108,8 +136,13 @@ int upscli_disconnect(UPSCONN *ups);
 #define UPSCLI_ERR_USERREQUIRED	35	/* Username required */
 #define UPSCLI_ERR_SSLFAIL	36	/* SSL is not available */
 #define UPSCLI_ERR_SSLERR	37	/* SSL error: %s */
+#define UPSCLI_ERR_SRVDISC	38	/* Server disconnected */
+#define UPSCLI_ERR_DRVNOTCONN	39	/* Driver not connected */
+#define UPSCLI_ERR_NOMEM	40	/* Memory allocation failure */
+#define UPSCLI_ERR_PARSE	41	/* Parse error: %s */
+#define UPSCLI_ERR_PROTOCOL	42	/* Protocol error */
 
-#define UPSCLI_ERR_MAX		37	/* stop here */
+#define UPSCLI_ERR_MAX		42	/* stop here */
 
 /* list types for use with upscli_getlist */
 
