@@ -382,6 +382,13 @@ static void draw_battpct(double var, int min, int nom, int max,
 	drawbar(0, 100, 1, 10, 20, 0, 50, -1, -1, 80, 100, var, format);
 }
 
+/* draws BATTVOLT bar style indicator */
+static void draw_battvolt(double var, int min, int nom, int max, 
+		int deviation, const char *format)
+{
+	drawbar(10, 40, 1, 5, 10, 0, 20, 28, 40, 25, 27, var, format);
+}
+
 /* draws UPSLOAD bar style indicator */
 static void draw_upsload(double var, int min, int nom, int max, 
 		int deviation, const char *format)
@@ -495,6 +502,8 @@ struct imgvar_t imgvar[] = {
 		"%.1f VAC",	draw_utility },
 	{ "battpct",	NULL,		NULL,		NULL,		0,	
 		"%.1f %%",	draw_battpct },
+      { "battvolt",   NULL,           NULL,           NULL,           0,      
+              "%.1f VDC",     draw_battvolt },
 	{ "upstemp",	"tempmin",	NULL,		"tempmax",	10,	
 		"%.1f C",	drawgeneralbar },
 	{ "acfreq",	NULL,		"nom_in_freq",	NULL,		2,	
