@@ -17,6 +17,20 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+/* Basic theory of operation:
+ *
+ * First we go through and parse as much of the status format string as
+ * possible.  We used to do this parsing run every time, but that's a 
+ * waste of CPU since it can't change during the program's run.
+ *
+ * This version does the parsing pass once, and creates a linked list of
+ * pointers to the functions that do the work and the arg they get.
+ * 
+ * That means the main loop just has to run the linked list and call
+ * anything it finds in there.  Everything happens from there, and we
+ * don't have to pointlessly reparse the string every time around.
+ */
+
 #include "common.h"
 #include "upsclient.h"
 
@@ -24,34 +38,39 @@
 #include "timehead.h"
 #include "upslog.h"
 
-	static	int	port;
+	static	int	port, reopen_flag = 0;
 	static	char	*upsname, *hostname;
 	static	UPSCONN	ups;
 
 	static	FILE	*logfile;
 	static	const	char *logfn, *monhost;
 	static	sigset_t	upsl_sigmask;
-	static	char	logformat[LARGEBUF], logbuffer[LARGEBUF];
+	static	char	*logformat, logbuffer[LARGEBUF];
 
-#define DEFAULT_LOGFORMAT "%TIME @Y@m@d @H@M@S% %VAR battpct% " \
-		"%VAR utility% %VAR loadpct% [%VAR status%] " \
-		"%VAR upstemp% %VAR acfreq%"
+	static	struct	flist_t	*fhead = NULL;
 
-void sig_hup(int signal)
+#define DEFAULT_LOGFORMAT "%TIME @Y@m@d @H@M@S% %VAR battery.charge% " \
+		"%VAR input.voltage% %VAR ups.load% [%VAR ups.status%] " \
+		"%VAR ups.temperature% %VAR input.frequency%"
+
+static void reopen_log(void)
 {
 	if (logfile == stdout) {
-		upslogx(LOG_INFO, "SIGHUP: logging to stdout");
+		upslogx(LOG_INFO, "logging to stdout");
 		return;
 	}
 
-	upslogx(LOG_INFO, "SIGHUP: Reopening log file");
-
 	fclose(logfile);
 	logfile = fopen(logfn, "a");
 	if (logfile == NULL)
 		fatal("could not reopen logfile %s", logfn);
+}
 
-	return;
+static void set_reopen_flag(int sig)
+{
+	upslogx(LOG_INFO, "Signal %d: reopening log file", sig);
+
+	reopen_flag = 1;
 }
 
 /* install handler for reloading config on SIGHUP */
@@ -62,23 +81,27 @@ static void instsighandler(void)
 	sigemptyset(&upsl_sigmask);
 	sigaddset(&upsl_sigmask, SIGHUP);
 	sa.sa_mask = upsl_sigmask;
-	sa.sa_handler = sig_hup;
+	sa.sa_handler = set_reopen_flag;
 	sa.sa_flags = 0;
 	if (sigaction(SIGHUP, &sa, NULL) < 0)
 		fatal("Can't install SIGHUP handler");
 }
 
-static void help(char *prog)
+static void help(const char *prog)
 {
-	printf("usage: %s <ups> <logfile> <interval> [format string]\n\n",
-		prog);
-
 	printf("UPS status logger.\n");
+
+	printf("\nusage: %s [OPTIONS]\n", prog);
 	printf("\n");
 
-	printf("  <ups>		- UPS to monitor - [<upsname>@]host[:port]\n");
-	printf("  <logfile>	- Log filename, or - for stdout\n");
-	printf("  <interval>	- Time between updates, in seconds\n");
+	printf("  -f <format>	- Log format.  See below for details.\n");
+	printf("		- Use -f \"<format>\" so your shell doesn't break it up.\n");
+	printf("  -i <interval>	- Time between updates, in seconds\n");
+	printf("  -l <logfile>	- Log file name, or - for stdout\n");
+	printf("  -s <ups>	- Monitor UPS <ups> - [<upsname>@]host[:port]\n");
+	printf("        	- Example: -s myups@server\n");
+	printf("  -u <user>	- Switch to <user> if started as root\n");
+
 	printf("\n");
 	printf("Some valid format string escapes:\n");
 	printf("\t%%%% insert a single %%\n");
@@ -91,7 +114,7 @@ static void help(char *prog)
 	printf("%s\n", DEFAULT_LOGFORMAT);
 
 	printf("\n");
-	printf("See the upslog(8) man page for more information\n");
+	printf("See the upslog(8) man page for more information.\n");
 
 	exit(0);
 }
@@ -143,21 +166,60 @@ static void do_time(const char *arg)
 	free(format);
 }
 
-static void do_var(const char *arg)
+static void old_getvar(const char *arg)
 {
 	char	varval[16];
-	int	res;
+	int	ret;
 
+	ret = upscli_getvar(&ups, upsname, arg, varval, sizeof(varval));
+
+	if (ret < 0)
+		snprintfcat(logbuffer, sizeof(logbuffer), "NA");
+	else
+		snprintfcat(logbuffer, sizeof(logbuffer), "%s", varval);
+}
+
+static void new_getvar(const char *var)
+{
+	int	ret, numq, numa;
+	const	char	*query[4];
+	char	**answer;
+
+	query[0] = "VAR";
+	query[1] = upsname;
+	query[2] = var;
+	numq = 3;
+
+	ret = upscli_get(&ups, numq, query, &numa, &answer);
+
+	if ((ret < 0) || (numa < numq)) {
+		snprintfcat(logbuffer, sizeof(logbuffer), "NA");
+		return;
+	}
+
+	snprintfcat(logbuffer, sizeof(logbuffer), "%s", answer[3]);
+}
+
+static void do_var(const char *arg)
+{
 	if ((!arg) || (strlen(arg) < 1)) {
 		snprintfcat(logbuffer, sizeof(logbuffer), "INVALID");
 		return;
 	}
 
-	res = upscli_getvar(&ups, upsname, arg, varval, sizeof(varval));
-	if (res < 0)
-		strcpy(varval, "NA");
+	/* old variable name? */
+	if (!strchr(arg, '.')) {
+		old_getvar(arg);
+		return;
+	}
 
-	snprintfcat(logbuffer, sizeof(logbuffer), "%s", varval);
+	/* no upsname = fall back on old way (until 2.0) */
+	if (!upsname) {
+		old_getvar(arg);
+		return;
+	} 
+
+	new_getvar(arg);
 }
 
 static void do_etime(const char *arg)
@@ -168,27 +230,64 @@ static void do_etime(const char *arg)
 	snprintfcat(logbuffer, sizeof(logbuffer), "%ld", (unsigned long) tod);
 }
 
-/* FUTURE: "compile" these results so it only needs to be parsed once */
-static void print_status(void)
+static void print_literal(const char *arg)
+{
+	snprintfcat(logbuffer, sizeof(logbuffer), "%s", arg);
+}
+
+/* register another parsing function to be called later */
+static void add_call(void (*fptr)(const char *arg), const char *arg)
+{
+	struct	flist_t	*tmp, *last;
+
+	tmp = last = fhead;
+
+	while (tmp) {
+		last = tmp;
+		tmp = tmp->next;
+	}
+
+	tmp = xmalloc(sizeof(struct flist_t));
+
+	tmp->fptr = fptr;
+
+	if (arg)
+		tmp->arg = xstrdup(arg);
+	else
+		tmp->arg = NULL;
+
+	tmp->next = NULL;
+
+	if (last)
+		last->next = tmp;	
+	else
+		fhead = tmp;
+}
+
+/* turn the format string into a list of function calls with args */
+static void compile_format(void)
 {
 	int	i, j, found, ofs;
 	char	*cmd, *arg, *ptr;
 
-	/* parse the format string */
-	memset(logbuffer, 0, sizeof(logbuffer));
-
 	for (i = 0; i < strlen(logformat); i++) {
 
 		/* if not a % sequence, append character and start over */
 		if (logformat[i] != '%') {
-			snprintfcat(logbuffer, sizeof(logbuffer), "%c", 
-				logformat[i]);
+			char	buf[4];
+
+			/* we have to stuff it into a string first */
+			snprintf(buf, sizeof(buf), "%c", logformat[i]);
+			add_call(print_literal, buf);
+
 			continue;
 		}
 
 		/* if a %%, append % and start over */
 		if (logformat[i+1] == '%') {
-			snprintfcat(logbuffer, sizeof(logbuffer), "%%");
+			add_call(print_literal, "%");
+
+			/* make sure we don't parse the second % next time */
 			i++;
 			continue;
 		}
@@ -200,7 +299,8 @@ static void print_status(void)
 
 		/* no trailing % = broken */
 		if (!ptr) {
-			snprintfcat(logbuffer, sizeof(logbuffer), "INVALID");
+			add_call(print_literal, "INVALID");
+			free(cmd);
 			continue;
 		}
 
@@ -221,7 +321,8 @@ static void print_status(void)
 		for (j = 0; logcmds[j].name != NULL; j++) {
 			if (strncasecmp(cmd, logcmds[j].name, 
 				strlen(logcmds[j].name)) == 0) {
-				logcmds[j].func(arg);
+
+				add_call(logcmds[j].func, arg);
 				found = 1;
 				break;
 			}
@@ -230,40 +331,121 @@ static void print_status(void)
 		free(cmd);
 
 		if (!found)
-			snprintfcat(logbuffer, sizeof(logbuffer), "INVALID");
+			add_call(print_literal, "INVALID");
 
 		/* now do the skip ahead saved from before */
 		i += ofs;
 
 	} /* for (i = 0; i < strlen(logformat); i++) */
-		
+}
+
+/* go through the list of functions and call them in order */
+static void run_flist(void)
+{
+	struct	flist_t	*tmp;
+
+	tmp = fhead;
+
+	memset(logbuffer, 0, sizeof(logbuffer));
+
+	while (tmp) {
+		tmp->fptr(tmp->arg);
+
+		tmp = tmp->next;
+	}
+
 	fprintf(logfile, "%s\n", logbuffer);
 	fflush(logfile);
 }
 
+	/* -s <monhost>
+	 * -l <log file>
+	 * -i <interval>
+	 * -f <format>
+	 * -u <username>
+	 */
+
 int main(int argc, char **argv)
 {
-	int	interval, i;
+	int	interval = 30, i;
+	char	*prog = NULL, *user = NULL;
+	struct	passwd	*new_uid = NULL;
 
-	snprintf(logformat, sizeof(logformat), "%s", DEFAULT_LOGFORMAT);
+	logformat = DEFAULT_LOGFORMAT;
+	user = RUN_AS_USER;
 
 	printf("Network UPS Tools upslog %s\n", UPS_VERSION);
 
-	if (argc < 4)
-		help(argv[0]);
+	prog = argv[0];
 
-	monhost = argv[1];
-	logfn = argv[2];
-	interval = atoi(argv[3]);
+	while ((i = getopt(argc, argv, "+hs:l:i:f:u:V")) != EOF) {
+		switch(i) {
+			case 'h':
+				help(prog);
+				break;
 
-	if (argc != 4) {
-		/* read out the remaining argv entries to the format string */
-		logformat[0] = logformat[sizeof(logformat)-1] = 0;
-		for (i = 4; i < argc; i++)
-			snprintfcat(logformat, sizeof(logformat), "%s ",
-				argv[i]);
+			case 's':
+				monhost = optarg;
+				break;
+
+			case 'l':
+				logfn = optarg;
+				break;
+
+			case 'i':
+				interval = atoi(optarg);
+				break;
+
+			case 'f':
+				logformat = optarg;
+				break;
+
+			case 'u':
+				user = optarg;
+				break;
+
+			case 'V':
+				exit(0);
+		}
 	}
 
+	argc -= optind;
+	argv += optind;
+
+	/* not enough args for the old way? */
+	if ((argc == 1) || (argc == 2))
+		help(prog);
+
+	/* see if it's being called in the old style - 3 or 4 args */
+
+	/* <system> <logfn> <interval> [<format>] */
+
+	if (argc >= 3) {
+		monhost = argv[0];
+		logfn = argv[1];
+		interval = atoi(argv[2]);
+	}
+
+	if (argc >= 4) {
+		/* read out the remaining argv entries to the format string */
+
+		logformat = xmalloc(LARGEBUF);
+		memset(logformat, '\0', LARGEBUF);
+
+		for (i = 3; i < argc; i++)
+			snprintfcat(logformat, LARGEBUF, "%s ", argv[i]);
+	}
+
+	if (!monhost)
+		fatalx("No UPS defined for monitoring - use -s <system>");
+
+	if (!logfn)
+		fatalx("No filename defined for logging - use -l <file>");
+
+	/* shouldn't happen */
+	if (!logformat)
+		fatalx("No format defined - but this should be impossible");
+
 	printf("logging status of %s to %s (%is intervals)\n", 
 		monhost, logfn, interval);
 
@@ -281,6 +463,12 @@ int main(int argc, char **argv)
 	if (logfile == NULL)
 		fatal("could not open logfile %s", logfn);
 
+	/* now drop root if we have it */
+	if ((new_uid = get_user_pwent(user)) == NULL)
+		fatal("getpwnam(%s)", user);
+
+	become_user(new_uid);
+
 	openlog("upslog", LOG_PID, LOG_FACILITY); 
 
 	if (logfile != stdout) {
@@ -290,15 +478,22 @@ int main(int argc, char **argv)
 
 	writepid("upslog");
 
+	compile_format();
+
 	for (;;) {
 
+		if (reopen_flag) {
+			reopen_log();
+			reopen_flag = 0;
+		}
+
 		/* reconnect if necessary */
-		if (ups.fd == -1) {
+		if (upscli_fd(&ups) == -1) {
 			upscli_disconnect(&ups);
 			upscli_connect(&ups, hostname, port, 0);
 		}
 
-		print_status();
+		run_flist();
 		sleep(interval);
 	}
 
