@@ -873,7 +873,7 @@ static void checkconf(void)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, errmsg);
+			upslogx(LOG_WARNING, "%s", errmsg);
 		}
 	}
 
