@@ -51,7 +51,7 @@ int init_communication (void);
 void do_status(void);
 int init_ups_data (void);
 void updateinfo (void);
-void get_belkin_field (char *,char *,int);
+void get_belkin_field(char *temp, char *data, size_t datalen, int n);
 int get_belkin_reply (char*);
 void send_belkin_command (char, const char *, const char *);
 int recvbinary (char *, int);
