@@ -38,7 +38,7 @@
 
    Known issues:
     - larger units (>= 1000VA) have a 24V battery, so the battery
-      voltage reported should be multiplied by 2 if the INFO_MODEL
+      voltage reported should be multiplied by 2 if the model
       string indicates such a larger unit.
     - load percentage is only measured when running on battery, and
       is reported as 0 when on line.  This seems to be a hardware
@@ -50,6 +50,8 @@
       correctly connected to the serial port).
  */
 
+#define DRV_VERSION "0.02"
+
 #include <sys/ioctl.h>
 #include "main.h"
 
@@ -77,13 +79,13 @@ etapro_get_response(const char *resp_type)
 	switch (cp[1]) {
 		/* Handle ASCII text responses directly here.  */
 	case 'R':
-		setinfo(INFO_MFR, "%s", cp + 2);
+		dstate_setinfo("ups.mfr", "%s", cp + 2);
 		return 0;
 	case 'S':
-		setinfo(INFO_MODEL, "%s", cp + 2);
+		dstate_setinfo("ups.model", "%s", cp + 2);
 		return 0;
 	case 'T':
-		setinfo(INFO_MFRDATE, "%s", cp + 2);
+		dstate_setinfo("ups.mfr.date", "%s", cp + 2);
 		return 0;
 	}
 	/* Handle all other responses as hexadecimal numbers.  */
@@ -153,40 +155,33 @@ etapro_set_off_timer(int seconds)
 	upslogx(LOG_ERR, "etapro_set_off_timer: error, status=0x%02x", x);
 }
 
-void
-instcmd(int auxcmd, int dlen, char *data)
+int instcmd(const char *cmdname, const char *extra)
 {
-	switch (auxcmd) {
-	case CMD_OFF:
+	if (!strcasecmp(cmdname, "load.off")) {
 		etapro_set_off_timer(1);
-		break;
-	case CMD_ON:
-		etapro_set_on_timer(1);
-		break;
-	case CMD_SHUTDOWN:
-		upsdrv_shutdown();
-		break;
-	default:
-		upslogx(LOG_INFO, "instcmd: unknown type 0x%04x", auxcmd);
+		return STAT_INSTCMD_HANDLED;
 	}
+
+	if (!strcasecmp(cmdname, "load.on")) {
+		etapro_set_on_timer(1);
+		return STAT_INSTCMD_HANDLED;
+	}
+
+	if (!strcasecmp(cmdname, "shutdown.return")) {
+		upsdrv_shutdown();
+		return STAT_INSTCMD_HANDLED;
+	}
+
+	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	return STAT_INSTCMD_UNKNOWN;
 }
 
 void
 upsdrv_initinfo(void)
 {
-	addinfo(INFO_MFR, "", 0, 0);
-	addinfo(INFO_MODEL, "", 0, 0);
-	addinfo(INFO_MFRDATE, "", 0, 0);
-
-	addinfo(INFO_STATUS, "", 0, 0);
-	addinfo(INFO_UTILITY, "", 0, 0);
-	addinfo(INFO_OUTVOLT, "", 0, 0);
-	addinfo(INFO_BATTVOLT, "", 0, 0);
-	addinfo(INFO_LOADPCT, "", 0, 0);
-
-	addinfo(INFO_INSTCMD, "", 0, CMD_OFF);
-	addinfo(INFO_INSTCMD, "", 0, CMD_ON);
-	addinfo(INFO_INSTCMD, "", 0, CMD_SHUTDOWN);
+	dstate_addcmd("load.off");
+	dstate_addcmd("load.on");
+	dstate_addcmd("shutdown.return");
 
 	/* First command after power on returns junk - ignore it.  */
 	upssend("RI\r");
@@ -195,7 +190,7 @@ upsdrv_initinfo(void)
 
 	upsdrv_updateinfo();
 
-	upsh.instcmd = instcmd;
+	upsh.new_instcmd = instcmd;
 }
 
 void
@@ -208,86 +203,110 @@ upsdrv_updateinfo(void)
 	upssend("RI\r");  /* identify */
 
 	x = etapro_get_response("SR");  /* manufacturer */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	x = etapro_get_response("SS");  /* model */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	x = etapro_get_response("ST");  /* mfr date */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	x = etapro_get_response("SU");  /* UPS ident */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 
 	upssend("RP\r");  /* read measurements */
 
 	x = etapro_get_response("SO");  /* status flags */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	flags = x;
 
 	x = etapro_get_response("SG");  /* input voltage, 0xFF = 270V */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	utility = (270.0 / 255) * x;
 
 	x = etapro_get_response("SH");  /* output voltage, 0xFF = 270V */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	outvolt = (270.0 / 255) * x;
 
 	x = etapro_get_response("SI");  /* battery voltage, 0xFF = 14V */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
+
 	/* TODO: >= 1000VA models have a 24V battery (max 28V) - check
 	   the model string returned by the RI command.  */
 	battvolt = (14.0 / 255) * x;
 
 	x = etapro_get_response("SL");  /* load (on battery), 0xFF = 150% */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	loadpct = (150.0 / 255) * x;
 
 	x = etapro_get_response("SN");  /* time running on battery */
-	if (x < 0)
+	if (x < 0) {
+		dstate_datastale();
 		return;
+	}
 	/* This is the time how long the UPS has been running on battery
 	   (in seconds, reset to zero after power returns), but there
-	   seems to be no INFO_xxx variable defined for this yet...  */
+	   seems to be no variable defined for this yet...  */
+
+	status_init();
 
 	status[0] = '\0';
 	if (!(flags & 0x02))
-		strcat(status, "OFF");
+		status_set("OFF");
 	else if (flags & 0x01)
-		strcat(status, "OL");
+		status_set("OL");
 	else
-		strcat(status, "OB");
+		status_set("OB");
 
 	if (!(flags & 0x04))
-		strcat(status, " LB");
+		status_set("LB");
 
 	/* TODO bit 3: 1 = ok, 0 = fault */
 
 	if (flags & 0x10)
-		strcat(status, " BOOST");
+		status_set("BOOST");
 
 	if (loadpct > 100.0)
-		strcat(status, " OVER");
+		status_set("OVER");
 
 	/* Battery voltage out of range (lower than LB, or too high).  */
 	if (flags & 0x20)
-		strcat(status, " RB");
+		status_set("RB");
 
 	/* TODO bit 6: 1 = charging, 0 = full */
 
-	setinfo(INFO_STATUS, "%s", status);
-	setinfo(INFO_UTILITY, "%03.1f", utility);
-	setinfo(INFO_OUTVOLT, "%03.1f", outvolt);
-	setinfo(INFO_BATTVOLT, "%02.2f", battvolt);
-	setinfo(INFO_LOADPCT, "%03.1f", loadpct);
+	status_commit();
 
-	writeinfo();
+	dstate_setinfo("input.voltage", "%03.1f", utility);
+	dstate_setinfo("output.voltage", "%03.1f", outvolt);
+	dstate_setinfo("battery.voltage", "%02.2f", battvolt);
+	dstate_setinfo("ups.load", "%03.1f", loadpct);
+
+	dstate_dataok();
 }
 
 /* TODO: delays should be tunable, the UPS supports max 32767 minutes.  */
@@ -318,7 +337,8 @@ upsdrv_makevartable(void)
 void
 upsdrv_banner(void)
 {
-	printf("Network UPS Tools - ETA PRO driver 0.01 (%s)\n\n", UPS_VERSION);
+	printf("Network UPS Tools - ETA PRO driver %s (%s)\n\n", 
+		DRV_VERSION, UPS_VERSION);
 }
 
 void
@@ -332,8 +352,6 @@ upsdrv_initups(void)
 	ioctl(upsfd, TIOCMBIS, &rts_bit);
 }
 
-int
-upsdrv_infomax(void)
+void upsdrv_cleanup(void)
 {
-	return 20;
 }
