@@ -69,19 +69,12 @@ char *GetTypeUpsName()
 
 void upsdrv_initinfo(void)
 {
-	addinfo (INFO_MFR, "Ever", 0, 0);
-	addinfo (INFO_MODEL, GetTypeUpsName(), 0, 0);
-	addinfo (INFO_STATUS, "", 0, 0);
-	addinfo (INFO_UTILITY, "",0,0);
-	addinfo (INFO_BATTPCT,"",0,0);
-/*	addinfo (INFO_LOWXFER,"",0,0);
-	addinfo (INFO_HIGHXFER,"",0,0);*/
-	addinfo (INFO_BATTVOLT,"",0,0);
+	dstate_setinfo("ups.mfr", "Ever");
+	dstate_setinfo("ups.model", "%s", GetTypeUpsName());
 }
 
 void upsdrv_updateinfo(void)
 {
-	char	temp[VALSIZE];
 	int	battery=0,standby=0;
 	unsigned char recBuf[2];
 	unsigned long acuV;
@@ -90,6 +83,7 @@ void upsdrv_updateinfo(void)
 	
 	if (!Code(2)) {
 		upslog(LOG_INFO, "Code failed");
+		dstate_datastale();
 		return;
 	}
 	/*Line status*/
@@ -106,6 +100,7 @@ void upsdrv_updateinfo(void)
 		acuV/=255;
 	} else {
 		upslog(LOG_INFO, "Code failed");
+		dstate_datastale();
 		return;
 	}
 	if (Code(1)) {  /*Line voltage*/
@@ -114,6 +109,7 @@ void upsdrv_updateinfo(void)
 		lineV=(recBuf[0]*100+recBuf[1]*25600)/372;
 	} else {
 		upslog(LOG_INFO, "Code failed");
+		dstate_datastale();
 		return;
 	}
 
@@ -129,10 +125,9 @@ void upsdrv_updateinfo(void)
 
 	status_commit();
 
-	snprintf(temp,VALSIZE,"%03ld",lineV);
-	setinfo (INFO_UTILITY,temp);
-	snprintf(temp,VALSIZE,"%03.2f",(double)acuV /10.0);
-	setinfo (INFO_BATTVOLT,temp);
+	dstate_setinfo("input.voltage", "%03ld", lineV);
+	dstate_setinfo("battery.voltage", "%03.2f", (double)acuV /10.0);
+
 	fVal=((double)acuV-95.0)*100.0;
 	if (standby)
 	  fVal/=(135.5-95.0);
@@ -142,10 +137,10 @@ void upsdrv_updateinfo(void)
 		fVal=100;
 	else if (fVal<0)
 		fVal=0;
-	snprintf(temp,VALSIZE,"%03.1f",fVal);
-	setinfo (INFO_BATTPCT,temp);
 
-	writeinfo();
+	dstate_setinfo("battery.charge", "%03.1f", fVal);
+
+	dstate_dataok();
 }
 
 void upsdrv_shutdown(void)
@@ -190,9 +185,6 @@ void upsdrv_initups(void)
 	InitUpsType();
 }
 
-/* tell main how many entries we need */
-int upsdrv_infomax(void)
+void upsdrv_cleanup(void)
 {
-        return 32;
 }
-
