@@ -20,6 +20,8 @@
 
 */
 
+#define DRV_VERSION "0.02"
+
 #define xDEBUG
 
 #ifdef DEBUG
@@ -38,23 +40,19 @@
 
 void upsdrv_initinfo(void)
 {
-	 addinfo(INFO_MFR, "Sola/Basic Mexico", 0, 0); 
-	 addinfo(INFO_MODEL, "SR-Inet 280/300/400/480/500/800", 0, 0); 
-	 addinfo(INFO_UTILITY, "", 0, 0); /* polled */
-	 addinfo(INFO_BATTPCT, "", 0, 0); /* polled */
-	 addinfo(INFO_ACFREQ, "", 0, 0);  /* polled */
-	 /* status conditions: OL, OB, LB, TRIM, BOOST, RB */
-	 addinfo(INFO_STATUS, "", 0, 0);  /* polled */
-	 /* high/low voltage */
-	 addinfo(INFO_LOWXFER, "95.0", 0, 0);  /* defined */
-	 addinfo(INFO_HIGHXFER, "145.0", 0, 0); /* defined */
+	dstate_setinfo("ups.mfr", "Sola/Basic Mexico");
+	dstate_setinfo("ups.model", "SR-Inet 280/300/400/480/500/800");
 
- 	 addinfo(INFO_OUTVOLT, "120.0", 0, 0); /* defined */
+	/* high/low voltage */
+	dstate_setinfo("input.transfer.low", "95.0");	/* defined */
+	dstate_setinfo("input.transfer.high", "145.0");	/* defined */
+
+	dstate_setinfo("output.voltage", "120.0");	/* defined */
 	 
- 	 addinfo(INFO_MAXUTIL, "", 0, 0); /* polled */
- 	 addinfo(INFO_MINUTIL, "", 0, 0); /* polled */
  	 /* addinfo(INFO_, "", 0, 0); */
 	 /*printf("Using %s %s on %s\n", getdata(INFO_MFR), getdata(INFO_MODEL), device_path);*/
+
+	dstate_setinfo("driver.version.internal", "%s", DRV_VERSION);
 }
 
 static char *getpacket(int *we_know){
@@ -147,7 +145,10 @@ void upsdrv_updateinfo(void)
   for (i=0;i<5;i++) {
 	  if ((buf=getpacket(&bytes_per_packet))!=NULL) break;
   }
-  if (!bytes_per_packet || !buf) return;
+  if (!bytes_per_packet || !buf) {
+	dstate_datastale();
+	return;
+  }
 	  
   /* do the parsing */
   {
@@ -163,29 +164,29 @@ void upsdrv_updateinfo(void)
      }
      snprintf(buf2,16,"%5.1f",in_volt);
      D(printf("utility=%s\n",buf2);)
-     setinfo(INFO_UTILITY, "%s", buf2);     
+     dstate_setinfo("input.voltage", "%s", buf2);     
      
      if (in_volt >= high_volt) high_volt=in_volt;
      snprintf(buf2,16,"%5.1f",high_volt);
      D(printf("highvolt=%s\n",buf2);)
-     setinfo(INFO_MAXUTIL, "%s", buf2);       
+     dstate_setinfo("input.voltage.maximum", "%s", buf2);       
      
      if (in_volt <= low_volt)  low_volt=in_volt;
      snprintf(buf2,16,"%5.1f",low_volt);
      D(printf("lowvolt=%s\n",buf2);)
-     setinfo(INFO_MINUTIL, "%s", buf2);      
+     dstate_setinfo("input.voltage.minimum", "%s", buf2);      
  
      battpct = (double)((unsigned char)buf[(bytes_per_packet==10)?5:4])*100.0/255.0;
      snprintf(buf2,16,"%5.1f",battpct);
      D(printf("battpct=%s\n",buf2);)
-     setinfo(INFO_BATTPCT, "%s", buf2);    
+     dstate_setinfo("battery.charge", "%s", buf2);    
  
      d=(unsigned char)buf[(bytes_per_packet==10)?6:5]*256
 	     + (unsigned char)buf[(bytes_per_packet==10)?7:6];    
      acfreq = 1000000/d;
      snprintf(buf2,16,"%5.2f",acfreq);
      D(printf("acfreq=%s\n",buf2);)
-     setinfo(INFO_ACFREQ, "%s", buf2);    
+     dstate_setinfo("input.frequency", "%s", buf2);    
 
      D(printf("status: ");)
      status_init();
@@ -211,10 +212,10 @@ void upsdrv_updateinfo(void)
      }
      D(printf("\n");)
      status_commit();	     
-     writeinfo();
 				  
 	     
    } 
+   dstate_dataok();
    return;
 }
 
@@ -250,8 +251,8 @@ void upsdrv_makevartable(void)
 
 void upsdrv_banner(void)
 {
-	printf("Network UPS Tools - ISBMEX UPS driver 0.01 (%s)\n\n",
-		       UPS_VERSION);
+	printf("Network UPS Tools - ISBMEX UPS driver %s (%s)\n\n",
+		DRV_VERSION, UPS_VERSION);
 }
 
 void upsdrv_initups(void)
@@ -261,8 +262,6 @@ void upsdrv_initups(void)
          flag_timeoutfailure = -1;
 }
 
-/* tell main how many items you need */
-int upsdrv_infomax(void)
+void upsdrv_cleanup(void)
 {
-	return 16;
 }
