@@ -18,14 +18,15 @@
 */
 
 #include "main.h"
+#include "dstate.h"
 
 	unsigned int sddelay = 90;	/* wait 90 seconds for shutdown */
 	const char *progname = NULL, *upsname = NULL;
 	const char *device_name = NULL, *device_path = NULL;
-	const char *statepath = NULL;
 	int	do_forceshutdown = 0;
 	vartab_t	*vartab_h = NULL;
 	int	experimental_driver = 0;
+	int	broken_driver = 0;
 
 	/* stuff from upscommon */
 	extern	int	do_lock_port;
@@ -37,6 +38,12 @@
 	/* for detecting -a values that don't match anything */
 	int	upsname_found = 0;
 
+	/* variables possibly set by the global part of ups.conf */
+	static unsigned int	poll_interval = 2;
+
+	/* set by upscommon's signal handlers */
+	int	exit_flag = 0;
+
 /* power down the attached load immediately */
 static void forceshutdown(void)
 {
@@ -54,32 +61,21 @@ static void forceshutdown(void)
 	exit(0);
 }
 
-void usage(void)
-{
-	printf ("usage: %s [-h] [-a <id>] [-d <num>] [-k] [-x <var>=<val>] [<device>]\n", 
-	        progname);
-	printf("\n");
-	printf("Examples:\n");
-	printf("         %s -a myups       - new method: use ups.conf section myups\n",
-		progname);
-	printf("         %s /dev/ttyS0     - old method: start up on port /dev/ttyS0\n", 
-		progname);
-	exit(1);
-}
-
-void help(void)
+static void help(void)
 {
 	vartab_t	*tmp;
 
-	printf ("usage: %s [-h] [-a <id>] [-d <num>] [-k] [-x <var>=<val>] [<device>]\n", 
-	        progname);
+	printf("\nusage: %s [OPTIONS] [<device>]\n\n", progname);
 
-	printf (
+	printf(
 		"  -a <id>        - autoconfig using ups.conf section <id>\n"
 		"  -d <num>       - wait <num> seconds after sending shutdown command\n"
 		"                 - note: -x and -d after -a override ups.conf settings\n"
-		"  -k             - force shutdown\n"
+		"  -D             - raise debugging level\n"
 		"  -h             - display this help\n"
+		"  -k             - force shutdown\n"
+		"  -r <dir>       - chroot to <dir>\n"
+		"  -u <user>      - switch to <user> (if started as root)\n"
 		"  -x <var>=<val> - set driver variable <var> to <val>\n"
 		"                 - example: -x cable=940-0095B\n"
 		"\n"
@@ -90,7 +86,7 @@ void help(void)
 	if (vartab_h) {
 		tmp = vartab_h;
 
-		printf ("Acceptable values for -x in this driver:\n\n");
+		printf("Acceptable values for -x or ups.conf in this driver:\n\n");
 
 		while (tmp) {
 			if (tmp->vartype == VAR_VALUE)
@@ -103,9 +99,28 @@ void help(void)
 	}
 
 	upsdrv_help();
+
 	exit(1);
 }
 
+/* store these in dstate as driver.(parameter|flag) */
+static void dparam_setinfo(const char *var, const char *val)
+{
+	char	vtmp[SMALLBUF];
+
+	/* store these in dstate for debugging and other help */
+	if (val) {
+		snprintf(vtmp, sizeof(vtmp), "driver.parameter.%s", var);
+		dstate_setinfo(vtmp, "%s", val);
+		return;
+	}
+
+	/* no value = flag */
+
+	snprintf(vtmp, sizeof(vtmp), "driver.flag.%s", var);
+	dstate_setinfo(vtmp, "enabled");
+}
+
 /* cram var [= <val>] data into storage */
 static void storeval(const char *var, char *val)
 {
@@ -130,6 +145,10 @@ static void storeval(const char *var, char *val)
 			if (val)
 				tmp->val = xstrdup(val);
 
+			/* don't keep things like SNMP community strings */
+			if ((tmp->vartype & VAR_SENSITIVE) == 0)
+				dparam_setinfo(var, val);
+
 			tmp->found = 1;
 			return;
 		}
@@ -137,8 +156,14 @@ static void storeval(const char *var, char *val)
 		tmp = tmp->next;
 	}
 
-	printf("Error: -x %s is not valid for this driver.\n\n", var);
-	help();
+	/* try to help them out */
+	printf("\nFatal error: '%s' is not a valid %s for this driver.\n", var,
+		val ? "variable name" : "flag");
+	printf("\n");
+	printf("Look in the man page or call this driver with -h for a list of\n");
+	printf("valid variable names and flags.\n");
+
+	exit(1);
 }
 
 /* retrieve the value of variable <var> if possible */
@@ -203,6 +228,7 @@ static int main_arg(char *var, char *val)
 
 	if (!strcmp(var, "nolock")) {
 		do_lock_port = 0;
+		dstate_setinfo("driver.flag.nolock", "enabled");
 		return 1;	/* handled */
 	}
 
@@ -215,11 +241,13 @@ static int main_arg(char *var, char *val)
 	if (!strcmp(var, "port")) {
 		device_path = xstrdup(val);
 		device_name = xbasename(device_path);
+		dstate_setinfo("driver.parameter.port", "%s", val);
 		return 1;	/* handled */
 	}
 
 	if (!strcmp(var, "sddelay")) {
 		sddelay = atoi(val);
+		dstate_setinfo("driver.parameter.sddelay", "%s", val);
 		return 1;	/* handled */
 	}
 
@@ -227,14 +255,32 @@ static int main_arg(char *var, char *val)
 	if (!strcmp(var, "sdorder"))
 		return 1;	/* handled */
 
+	/* only for upsd (at the moment) - ignored here */
+	if (!strcmp(var, "desc"))
+		return 1;       /* handled */
+
 	return 0;	/* unhandled, pass it through to the driver */
 }
 
+static void do_global_args(const char *var, const char *val)
+{
+	if (!strcmp(var, "pollinterval")) {
+		poll_interval = atoi(val);
+		return;
+	}
+
+	/* unrecognized */
+}
+
 void do_upsconf_args(char *confupsname, char *var, char *val)
 {
+	char	tmp[SMALLBUF];
+
 	/* ignore global stuff in here */
-	if (!confupsname)
+	if (!confupsname) {
+		do_global_args(var, val);
 		return;
+	}
 
 	/* no match = not for us */
 	if (strcmp(confupsname, upsname) != 0)
@@ -247,6 +293,11 @@ void do_upsconf_args(char *confupsname, char *var, char *val)
 
 	/* flags (no =) now get passed to the driver-level stuff */
 	if (!val) {
+
+		/* also store this, but it's a bit different */
+		snprintf(tmp, sizeof(tmp), "driver.flag.%s", var);
+		dstate_setinfo(tmp, "enabled");
+
 		storeval(var, NULL);
 		return;
 	}
@@ -289,123 +340,6 @@ static void splitxarg(char *inbuf)
 	storeval(buf, val);
 }
 
-/* two pipes for two different sync points during startup */
-
-#define SERIALIZE_INIT 1
-#define SERIALIZE_SETM1 2
-#define SERIALIZE_SETM2 3
-#define SERIALIZE_WAITM1 4
-#define SERIALIZE_WAITM2 5
-
-/* 'serialize' conflicts with a library function on HP-UX */
-void main_serialize(int op)
-{
-	static	int	pipe1[2], pipe2[2];
-	int	ret;
-	char	ch;
-
-	switch (op) {
-
-		case SERIALIZE_INIT:
-			ret = pipe(pipe1);
-
-			if (ret != 0) {
-				fatal("serialize: pipe");
-				exit(1);
-			}
-
-			ret = pipe(pipe2);
-
-			if (ret != 0) {
-				fatal("serialize: pipe");
-				exit(1);
-			}
-
-			break;
-
-		case SERIALIZE_SETM1:
-			close(pipe1[0]);
-			close(pipe1[1]);
-			break;
-
-		case SERIALIZE_WAITM1:
-			close(pipe1[1]);
-			ret = read(pipe1[0], &ch, 1);
-			close(pipe1[0]);
-			break;
-
-		case SERIALIZE_SETM2:
-			close(pipe2[0]);
-			close(pipe2[1]);
-			break;
-
-		case SERIALIZE_WAITM2:
-			close(pipe2[1]);
-			ret = read(pipe2[0], &ch, 1);
-			close(pipe2[0]);
-			break;
-	}
-}
-
-static void runparent(void)
-{
-	close_statefd();
-
-	/* allow the client to exit from its WAITM1 call */
-	main_serialize(SERIALIZE_SETM1);
-
-	/* wait for the client to call info_ready() and set M2 */
-	main_serialize(SERIALIZE_WAITM2);
-
-	_exit(0);
-}
-
-
-/* special version to deal with the nasty sync issues in here */
-static void main_background(void)
-{
-	int	pid;
-
-	if ((pid = fork()) < 0)
-		fatal("Unable to enter background");
-
-	xbit_set(&upslog_flags, UPSLOG_SYSLOG);
-	xbit_clear(&upslog_flags, UPSLOG_STDERR);
-
-	close(0);
-	close(1);
-	close(2);
-
-	if (pid != 0)
-		runparent();
-
-	/* child */
-
-	/* make fds 0-2 point somewhere defined */
-	if (open("/dev/null", O_RDWR) != 0)
-		fatal("open /dev/null");
-	dup(0);
-	dup(0);
-
-#ifdef HAVE_SETSID
-	setsid();		/* make a new session to dodge signals */
-#endif
-
-	/* wait until after parent closes the statefd */
-	main_serialize(SERIALIZE_WAITM1);
-
-	/* load up info with a full set of data */
-	upsdrv_updateinfo();
-
-	/* put the state file where it's supposed to be */
-	info_ready();
-
-	/* let the parent exit now that we're ready to roll */
-	main_serialize(SERIALIZE_SETM2);
-
-	upslogx(LOG_INFO, "Startup successful");
-}
-
 /* dump the list from the vartable for external parsers */
 void listxarg(void)
 {
@@ -432,14 +366,12 @@ void listxarg(void)
 
 int main(int argc, char **argv)
 {
+	char	*chroot_path = NULL, *user = NULL;
+	struct	passwd	*new_uid = NULL;
 	int	i;
-#ifdef HAVE_MMAP
-	int 	shmok = 0;
-#else
-	int 	shmok = 1;
-#endif
 
-	droproot();
+	/* pick up a default from configure --with-user */
+	user = RUN_AS_USER;
 
 	upsdrv_banner();
 
@@ -449,15 +381,12 @@ int main(int argc, char **argv)
 	}
 
 	progname = xbasename(argv[0]);
-	openlog(progname, LOG_PID, LOG_FACILITY);
-
-	if ((statepath = getenv("NUT_STATEPATH")) == NULL)
-		statepath = STATEPATH;
+	open_syslog(progname);
 
 	/* build the driver's extra (-x) variable table */
 	upsdrv_makevartable();
 
-	while ((i = getopt(argc, argv, "+a:d:kDhx:L")) != EOF) {
+	while ((i = getopt(argc, argv, "+a:d:kDhx:Lr:u:Vi:")) != EOF) {
 		switch (i) {
 			case 'a':
 				upsname = optarg;
@@ -471,27 +400,34 @@ int main(int argc, char **argv)
 			case 'd':
 				sddelay = atoi(optarg);
 				break;
+			case 'D':
+				nut_debug_level++;
+				break;
+			case 'i':
+				poll_interval = atoi(optarg);
+				break;
 			case 'k':
 				do_lock_port = 0;
 				do_forceshutdown = 1;
 				break;
-			case 'D':
-				nut_debug_level++;
+			case 'L':
+				listxarg();
+				exit(0);
+			case 'r':
+				chroot_path = optarg;
+				break;
+			case 'u':
+				user = optarg;
 				break;
 			case 'V':
-				printf("Network UPS Tools (%s)\n", UPS_VERSION);
+				/* already printed the banner, so exit */
 				exit(0);
 			case 'x':
 				splitxarg(optarg);
 				break;
-			case 'L':
-				listxarg();
-				exit(0);
 			case 'h':
-				help();
-				break;
 			default:
-				usage();
+				help();
 				break;
 		}
 	}
@@ -504,7 +440,7 @@ int main(int argc, char **argv)
 	if (argc < 1) {
 		if (!device_path) {
 			fprintf(stderr, "Error: You must specify a port name in ups.conf or on the command line.\n");
-			usage();
+			help();
 		}
 	}
 
@@ -514,45 +450,73 @@ int main(int argc, char **argv)
 		device_name = xbasename(device_path);
 	}
 
-	snprintf(statefn, sizeof(statefn), "%s/%s-%s", 
-		statepath, progname, device_name);
-
 	pidfn = xmalloc(SMALLBUF);
+
 	snprintf(pidfn, SMALLBUF, "%s/%s-%s.pid", 
-		ALTPIDPATH, progname, device_name);
+		altpidpath(), progname, device_name);
 
 	upsdebugx(1, "debug level is '%d'", nut_debug_level);
 
 	upsdrv_initups();
 
+	/* now see if things are very wrong out there */
+	if (broken_driver) {
+		printf("Fatal error: broken driver.  It probably needs to be converted.\n");
+		printf("Search for 'broken_driver = 1' in the source for more details.\n");
+		exit(1);
+	}
+
 	if (do_forceshutdown)
 		forceshutdown();
 
-	if (chdir(statepath))
-		fatal("Can't chdir to %s", statepath);
+	if ((new_uid = get_user_pwent(user)) == NULL)
+		fatal("getpwnam(%s)", user);
+	
+	if (chroot_path)
+		chroot_start(chroot_path);
 
-	create_info(upsdrv_infomax(), shmok);
-	createmsgq();
+	become_user(new_uid);
 
+	if (chdir(dflt_statepath()))
+		fatal("Can't chdir to %s", dflt_statepath());
+
+	setup_signals();
+
+	/* clear out callback handler data */
+	memset(&upsh, '\0', sizeof(upsh));
+
+	/* get the base data established before allowing connections */
 	upsdrv_initinfo();
+	upsdrv_updateinfo();
+
+	/* now we can start servicing requests */
+        dstate_init(progname, device_name);
+
+	/* publish the top-level data: version number, driver name */
+	dstate_setinfo("driver.version", "%s", UPS_VERSION);
+	dstate_setinfo("driver.name", "%s", progname);
 
 	if (nut_debug_level == 0) {
-		main_serialize(SERIALIZE_INIT);
-		main_background();
+		background();
 		writepid(pidfn);
-	} else {
-
-		/* no sync to worry about, so rename it and get going */
-		info_ready();
-
 	}
 
+	/* safe to do this now that the parent has exited */
+	atexit(exit_cleanup);
+
 	for (;;) {
 		upsdrv_updateinfo();
 
-		if (getupsmsg(2))       
-			upslogx(LOG_DEBUG, "Received a message from upsd");
+		/* FUTURE: make this delay configurable */
+		dstate_poll_fds(poll_interval);
+
+		/* deal with flags from the signal handlers */
+		if (exit_flag == 1) {
+			upsdrv_cleanup();
+			exit(0);
+		}
 	}
 
+	/* NOTREACHED */
 	return 0;
 }
