@@ -36,7 +36,7 @@
 /* --------------------------------------------------------------- */
 
 #define DRIVER_NAME    "MGE UPS SYSTEMS/U-Talk driver"
-#define DRIVER_VERSION "0.80.0"
+#define DRIVER_VERSION "0.81.0"
 
 /* delay after sending each char to UPS (in MICROSECONDS) */
 #define MGE_CHAR_DELAY 0
@@ -54,8 +54,6 @@
 #define MAXTRIES    10 /* max number of connect tries              */
 #define BUFFLEN    256 
 
-extern int  sddelay;         /* from main.c --- ignored for now  */
-
 #define SD_RETURN	0
 #define SD_STAYOFF	1
 
@@ -79,12 +77,12 @@ static struct {
 
 static int instcmd(const char *cmdname, const char *extra);
 static int setvar(const char *varname, const char *val);
-static void enable_ups_comm();
+static void enable_ups_comm(void);
 static void extract_info(const char *buf, const mge_info_item *mge, 
 			 char *infostr, int infolen);
-static char *info_variable_cmd(const char *type);
+static const char *info_variable_cmd(const char *type);
 static bool info_variable_ok(const char *type);
-static int  get_ups_status();
+static int  get_ups_status(void);
 static int mge_command(char *reply, int replylen, const char *fmt, ...);
 static void format_model_name(char *model);
 
@@ -118,6 +116,8 @@ void upsdrv_makevartable(void)
 		"Delay before shutdown, in seconds    (default = %3d)",
 		DEFAULT_OFFDELAY);
 	addvar (VAR_VALUE, "OffDelay", temp);
+
+	addvar(VAR_FLAG, "oldmac", "Enable Oldworld Apple Macintosh support");
 }
 
 /* --------------------------------------------------------------- */
@@ -126,10 +126,9 @@ void upsdrv_initups(void)
 {
 	char buf[BUFFLEN];
 	int RTS = TIOCM_RTS;
+	int  bytes_rcvd;
 	
 	open_serial(device_path, B2400);
-	ioctl(upsfd, TIOCMBIC, &RTS);
-	enable_ups_comm();
 
 	/* read command line/conf variables */
 	if (getval ("LowBatt"))
@@ -141,21 +140,41 @@ void upsdrv_initups(void)
 	if (getval ("OffDelay"))
 		mge_ups.OffDelay = atoi (getval ("OffDelay"));
 
-	/* set values given on the command line */
-	mge_command(buf, sizeof(buf), "Bl %d",  mge_ups.LowBatt);
-	if(strcmp(buf, "OK"))
-		upsdebugx(1, "UPS response to %d%% Low Batt Level was %s",
-			mge_ups.LowBatt, buf);
+	if (testvar ("oldmac"))
+		RTS = ~TIOCM_RTS;
+
+	/* Init serial line */
+	ioctl(upsfd, TIOCMBIC, &RTS);
+	enable_ups_comm();
 	
-	mge_command(buf, sizeof(buf), "Sm %d",  mge_ups.OnDelay);
-	if(strcmp(buf, "OK"))
-		upsdebugx(1, "UPS response to %d min ON delay was %s",
-			mge_ups.OnDelay, buf);
+	/* Get and set values (if exists) given on the command line */
+	bytes_rcvd = mge_command(buf, sizeof(buf), "Bl ?");
+	if(bytes_rcvd > 0 && buf[0] != '?') {
+		mge_command(buf, sizeof(buf), "Bl %d",  mge_ups.LowBatt);
+		if(strcmp(buf, "OK"))
+			upsdebugx(1, "UPS response to %d%% Low Batt Level was %s",
+				mge_ups.LowBatt, buf);
+	} else
+		upsdebugx(1, "initups: LowBatt unavailable");
 	
-	mge_command(buf, sizeof(buf), "Sn %d",  mge_ups.OffDelay);
-	if(strcmp(buf, "OK"))
-		upsdebugx(1, "UPS response to %d min OFF delay was %s",
-			mge_ups.OffDelay, buf);
+	bytes_rcvd = mge_command(buf, sizeof(buf), "Sm ?");
+	if(bytes_rcvd > 0 && buf[0] != '?') {
+		mge_command(buf, sizeof(buf), "Sm %d",  mge_ups.OnDelay);
+		if(strcmp(buf, "OK"))
+			upsdebugx(1, "UPS response to %d min ON delay was %s",
+				mge_ups.OnDelay, buf);
+	} else
+		upsdebugx(1, "initups: OnDelay unavailable");
+	
+	bytes_rcvd = mge_command(buf, sizeof(buf), "Sn ?");
+	if(bytes_rcvd > 0 && buf[0] != '?') {
+		mge_command(buf, sizeof(buf), "Sn %d",  mge_ups.OffDelay);
+		if(strcmp(buf, "OK"))
+			upsdebugx(1, "UPS response to %d min OFF delay was %s",
+				mge_ups.OffDelay, buf);
+	} else
+		upsdebugx(1, "initups: OffDelay unavailable");
+
 }
 
 /* --------------------------------------------------------------- */
@@ -250,11 +269,17 @@ void upsdrv_initinfo(void)
 			item->ok = TRUE;
 			extract_info(buf, item, infostr, sizeof(infostr));
 			dstate_setinfo(item->type, "%s", infostr);
+			dstate_setflags(item->type, item->flags);
 			upsdebugx(1, "initinfo: %s == >%s<", item->type, infostr);
+
+			/* Set max length for strings */
+			if (item->flags & ST_FLAG_STRING)
+				dstate_setaux(item->type, item->length);
 		}
 	} /* for item */
 
 	/* commands ----------------------------------------------- */
+	/* FIXME: check if available before adding! */
 	dstate_addcmd("load.off");
 	dstate_addcmd("load.on");
 	dstate_addcmd("shutdown.return");
@@ -335,9 +360,6 @@ void upsdrv_shutdown(void)
 
 void upsdrv_help(void)
 {
-	printf("\n");
-	printf("Option -d is ignored.\n");
-	printf("\n");
 }
 
 /* --------------------------------------------------------------- */
@@ -472,7 +494,7 @@ int setvar(const char *varname, const char *val)
     
 		/* Execute command */
 		mge_command(temp, sizeof(temp), cmd);
-		upsdebugx(1, "setvar: UPS response to Set %s to %s was %s", varname, val, temp);
+		upslogx(LOG_INFO, "setvar: UPS response to Set %s to %s was %s", varname, val, temp);
 	} else
 		upsdebugx(1, "setvar: Variable %s not supported by UPS", varname);
 	
@@ -481,7 +503,7 @@ int setvar(const char *varname, const char *val)
 /* --------------------------------------------------------------- */
 
 /* enable communication with UPS */
-static void enable_ups_comm()
+static void enable_ups_comm(void)
 {
 	mge_command(NULL, 0, "Z");   /* send Z twice --- speeds up re-connect */
 	mge_command(NULL, 0, "Z");   
@@ -532,7 +554,7 @@ static void extract_info(const char *buf, const mge_info_item *item,
 	 the elements of the second item are in buf[16..9].
 */
 
-static int get_ups_status()
+static int get_ups_status(void)
 {
 	char buf[BUFFLEN]; 
 	int rb_set= FALSE;  /* has RB flag been set ? */
@@ -603,9 +625,12 @@ static int get_ups_status()
 		} /* if strlen */      
 
 		if ( strlen(buf) > 15 ) {   /* second "byte", skip <SP> */
-			if (buf[16] == '1')
-				status_set("FSD");
+			if (buf[16] == '1') {
+				status_set("OB");
+				status_set("LB");
+			}
 
+				
 			if ( !(buf[9] == '1') )
 				status_set("OFF");
 		} /* if strlen */  
@@ -643,7 +668,7 @@ static bool info_variable_ok(const char *type)
 
 /* return proper variable "cmd" given INFO_ type */
    
-static char *info_variable_cmd(const char *type) 
+static const char *info_variable_cmd(const char *type) 
 {
 	mge_info_item *item = mge_info ;
 	
@@ -665,15 +690,19 @@ static char *info_variable_cmd(const char *type)
 */
 static int mge_command(char *reply, int replylen, const char *fmt, ...)
 {
-	char *p;
+	const char *p;
 	char command[BUFFLEN];
 	int bytes_sent = 0;
 	int bytes_rcvd = 0;
+	int ret;
 	va_list ap;
 	
 	/* build command string */
 	va_start(ap, fmt);
-	if (vsnprintf(command, sizeof(command), fmt, ap) >= sizeof(command))
+
+	ret = vsnprintf(command, sizeof(command), fmt, ap);
+
+	if ((ret < 1) || (ret >= (int) sizeof(command)))
 		upsdebugx(4, "mge_command: command truncated");
 	
 	va_end(ap);
