@@ -36,6 +36,8 @@
  * 20011208/PB: add support for subtypes
  * 20020629/SR: add support for calc. parameters
  * 20020701/SK: subtypes as structures
+ * 20030506/SK: replaced the previous validation method with
+ *              validation_byte[] and replaced subtype by type.
  */
 
 /* C-libary includes */
@@ -52,7 +54,7 @@
 
 /* general constants */
 enum general {
-       	SEC_FOR_POWERKILL         = 10U,
+	SEC_FOR_POWERKILL         = 10U,
 	MAX_NUM_OF_BYTES_FROM_UPS = 16U,
 };
 
@@ -65,50 +67,62 @@ enum commands {
 
 /* location of data in received string */
 enum data {
-	LOAD_LEVEL  = 0U,
-       	BAT_LEVEL   = 1U,
-       	IN_AC_VOLT  = 2U,
-       	OUT_AC_VOLT = 3U,
-	IN_AC_FREQ  = 4U,
-        OUT_AC_FREQ = 6U,
+	UPS_LOAD         = 0U,
+	BATTERY_CHARGE   = 1U,
+	INPUT_VOLTAGE    = 2U,
+	OUTPUT_VOLTAGE   = 3U,
+	INPUT_FREQUENCY  = 4U,
+	OUTPUT_FREQUENCY = 6U,
 };
 
 /* status bits */
 enum status {
 	SUMMARY       = 0U,
-       	MAINS_FAILURE = 1U,
-       	LOW_BAT       = 2U, 
+	MAINS_FAILURE = 1U,
+	LOW_BAT       = 2U, 
 	BAD_BAT       = 2U,
-       	TEST          = 4U,
+	TEST          = 4U,
 	AVR_ON        = 8U,
-       	OVERLOAD      = 32U,
-       	OFF           = 128U,
+	OVERLOAD      = 32U,
+	OFF           = 128U,
 	STATUS_A      = 9U,
 	STATUS_B      = 10U,
 };
 
-/* supported subtypes */
-struct subtype {
+/* supported types */
+struct type {
 	char          *name;
 	unsigned char num_of_bytes_from_ups;
 	void          (*setup_flow_control)(void);
-	int           (*validate_raw_data)(void);
+	
+	struct validation_byte {
+	    unsigned char index_of_byte, required_value;
+	    /* An example might explain the intention better then prose.
+	     * Suppose we want to validate the data with:
+	     *     powercom_raw_data[5] == 0x80
+	     * then we will set index_of_byte to 5 and required_value to
+	     * 0x80: { 5, 0x80 }.
+	     */
+	} validation[3];
+	/* The validation array is of length 3 because 3 is longest 
+	 * validation sequence for any type.
+	 */
 	
 	/* parameters to calculate input and output freq., one pair for each
-	 * subtype:
+	 * type:
 	 *  Each pair defines parameters for 1/(A*x+B) to calculate freq.
 	 *  from raw data
 	 */
 	float         freq[2];
 	
-	/* parameters to calculate load %, two pairs for each subtype:
+	/* parameters to calculate load %, two pairs for each type:
 	 *  First pair defines the parameters for A*x+B to calculate load
 	 *  from raw data when offline and the second pair is used when
 	 *  online
 	 */
 	float         loadpct[4];
 	
-	/* parameters to calculate battery %, five parameters for each subtype:
+	/* parameters to calculate battery %, five parameters for each type:
 	 *  First three params defines the parameters for A*x+B*y+C to calculate
 	 *  battery % (x is raw data, y is load %) when offline.
 	 *  Fourth and fifth parameters are used to calculate D*y+E when online.
@@ -116,7 +130,7 @@ struct subtype {
 	float         battpct[5];
 
 	/* parameters to calculate utility and output voltage, two pairs for
-	 * each subtype:
+	 * each type:
 	 *  First pair defines the parameters for A*x+B to calculate utility
 	 *  from raw data when line voltage is >=220 and the second pair
 	 *  is used otherwise.
