@@ -7,8 +7,6 @@
  * Tested on AP9606 APC Web/SNMP managenent card.
  * Dmitry Frolov <frolov@riss-telecom.ru>
  *
- * $RISS: nut/drivers/powernet.c,v 1.4 2002/10/21 07:19:39 frol Exp $
- *
  * Based on NUT snmp-ups driver:
  * Copyright (C) 2002 Arnaud Quette <arnaud.quette@free.fr>
  * some parts are Copyright (C) :
@@ -69,12 +67,10 @@
 #include "main.h"
 #include "powernet.h"
 
-static char upsdrv_version[] = PN_UPSDRV_VERSION;
-static char upsdrv_cvsid[] = "$RISS: nut/drivers/powernet.c,v 1.4 2002/10/21 07:19:39 frol Exp $";
-
 unsigned long g_ups_status[PN_STATUS_NUM_ELEM];
+struct snmp_session g_snmp_sess, *g_snmp_sess_p = NULL;
 
-struct snmp_session g_snmp_sess, *g_snmp_sess_p;
+extern char *upsname;
 
 /* -----------------------------------------------------------
  * SNMP functions.
@@ -97,7 +93,7 @@ void pn_snmp_init(const char *type, const char *hostname, const char *community)
 	SOCK_STARTUP;
 	g_snmp_sess_p = snmp_open(&g_snmp_sess);	/* establish the session */
 	if (g_snmp_sess_p == NULL) {
-		pn_snmp_perror(&g_snmp_sess, "pn_snmp_init: snmp_open");
+		pn_snmp_perror(&g_snmp_sess, 0, NULL, "pn_snmp_init: snmp_open");
 		exit(1);
 	}
 }
@@ -105,8 +101,10 @@ void pn_snmp_init(const char *type, const char *hostname, const char *community)
 void pn_snmp_cleanup(void)
 {
 	/* close snmp session. */
-	if (g_snmp_sess_p)
+	if (g_snmp_sess_p) {
 		snmp_close(g_snmp_sess_p);
+		g_snmp_sess_p = NULL;
+	}
 	SOCK_CLEANUP;
 }
 
@@ -120,8 +118,8 @@ struct snmp_pdu *pn_snmp_get(const char *OID)
 	/* create and send request. */
 
 	if (!read_objid(OID, name, &name_len)) {
-		upslogx(LOG_ERR, "pn_snmp_get: %s: %s",
-			OID, snmp_api_errstring(snmp_errno));
+		upslogx(LOG_ERR, "[%s] pn_snmp_get: %s: %s",
+			upsname, OID, snmp_api_errstring(snmp_errno));
 		return NULL;
 	}
 
@@ -135,7 +133,7 @@ struct snmp_pdu *pn_snmp_get(const char *OID)
 
 	if (!((status == STAT_SUCCESS) && (response->errstat == SNMP_ERR_NOERROR)))
 	{
-		pn_snmp_perror(g_snmp_sess_p, "pn_snmp_get: %s", OID);
+		pn_snmp_perror(g_snmp_sess_p, status, response, "pn_snmp_get: %s", OID);
 		snmp_free_pdu(response);
 		response = NULL;
 	}
@@ -165,12 +163,15 @@ bool pn_snmp_get_str(const char *OID, char *buf, size_t buf_len)
 		break;
 	case ASN_INTEGER:
 	case ASN_GAUGE:
-	case ASN_TIMETICKS:
 		len = snprintf(buf, buf_len, "%ld", *pdu->variables->val.integer);
 		break;
+	case ASN_TIMETICKS:
+		/* convert timeticks to seconds */
+		len = snprintf(buf, buf_len, "%ld", *pdu->variables->val.integer / 100);
+		break;
 	default:
-		upslogx(LOG_ERR, "unhandled ASN 0x%x recieved from %s",
-			pdu->variables->type, OID);
+		upslogx(LOG_ERR, "[%s] unhandled ASN 0x%x recieved from %s",
+			upsname, pdu->variables->type, OID);
 		return FALSE;
 		break;
 	}
@@ -201,12 +202,15 @@ bool pn_snmp_get_int(const char *OID, long *pval)
 		break;
 	case ASN_INTEGER:
 	case ASN_GAUGE:
-	case ASN_TIMETICKS:
 		value = *pdu->variables->val.integer;
 		break;
+	case ASN_TIMETICKS:
+		/* convert timeticks to seconds */
+		value = *pdu->variables->val.integer / 100;
+		break;
 	default:
-		upslogx(LOG_ERR, "unhandled ASN 0x%x recieved from %s",
-			pdu->variables->type, OID);
+		upslogx(LOG_ERR, "[%s] unhandled ASN 0x%x recieved from %s",
+			upsname, pdu->variables->type, OID);
 		return FALSE;
 		break;
 	}
@@ -227,25 +231,29 @@ bool pn_snmp_set(const char *OID, char type, const char *value)
 	oid name[MAX_OID_LEN];
 	size_t name_len = MAX_OID_LEN;
 
-	upsdebugx(2, "in pn_snmp_set");
-
 	if (!read_objid(OID, name, &name_len)) {
-		upslogx(LOG_ERR, "pn_snmp_set: %s: %s",
-			OID, snmp_api_errstring(snmp_errno));
+		upslogx(LOG_ERR, "[%s] pn_snmp_set: %s: %s",
+			upsname, OID, snmp_api_errstring(snmp_errno));
 		return FALSE;
 	}
 
 	pdu = snmp_pdu_create(SNMP_MSG_SET);
 	if (pdu == NULL)
 		fatalx("Not enough memory");
-	snmp_add_var(pdu, name, name_len, type, value);
+
+	if (snmp_add_var(pdu, name, name_len, type, value)) {
+		upslogx(LOG_ERR, "[%s] pn_snmp_set: %s: %s",
+			upsname, OID, snmp_api_errstring(snmp_errno));
+		return FALSE;
+	}
 
 	status = snmp_synch_response(g_snmp_sess_p, pdu, &response);
 
 	if ((status == STAT_SUCCESS) && (response->errstat == SNMP_ERR_NOERROR))
 		ret = TRUE;
 	else
-		pn_snmp_perror(g_snmp_sess_p, "pn_snmp_set: %s", OID);
+		pn_snmp_perror(g_snmp_sess_p, status, response,
+			"pn_snmp_set: can't set %s", OID);
 
 	snmp_free_pdu(response);
 
@@ -268,8 +276,20 @@ bool pn_snmp_set_int(const char *OID, long value)
 	return ret;
 }
 
+bool pn_snmp_set_time(const char *OID, long value)
+{
+	bool ret;
+	char *buf = xmalloc(PN_BUFSIZE);
+
+	snprintf(buf, PN_BUFSIZE, "%ld", value * 100);
+	ret = pn_snmp_set(OID, 't', buf);
+	free(buf);
+	return ret;
+}
+
 /* log descriptive SNMP error message. */
-void pn_snmp_perror(struct snmp_session *sess, const char *fmt, ...)
+void pn_snmp_perror(struct snmp_session *sess, int status,
+	struct snmp_pdu *response, const char *fmt, ...)
 {
 	va_list va;
 	int cliberr, snmperr;
@@ -282,10 +302,24 @@ void pn_snmp_perror(struct snmp_session *sess, const char *fmt, ...)
 	vsnprintf(buf, PN_LARGEBUF, fmt, va);
 	va_end(va);
 
-	snmp_error(sess, &cliberr, &snmperr, &snmperrstr);
-	upslogx(LOG_ERR, "%s: %s",
-		buf, snmperrstr);
-	free(snmperrstr);
+	if (response == NULL) {
+		snmp_error(sess, &cliberr, &snmperr, &snmperrstr);
+		upslogx(LOG_ERR, "[%s] %s: %s",
+			upsname, buf, snmperrstr);
+		free(snmperrstr);
+	} else if (status == STAT_SUCCESS) {
+		if (response->errstat != SNMP_ERR_NOERROR)
+			upslogx(LOG_ERR, "[%s] %s: Error in packet: %s",
+				upsname, buf, snmp_errstring(response->errstat));
+	} else if (status == STAT_TIMEOUT) {
+		upslogx(LOG_ERR, "[%s] %s: Timeout: no response from %s",
+			upsname, sess->peername);
+	} else {
+		snmp_sess_error(sess, &cliberr, &snmperr, &snmperrstr);
+		upslogx(LOG_ERR, "[%s] %s: %s",
+			upsname, buf, snmperrstr);
+		free(snmperrstr);
+	}
 
 	free(buf);
 }
@@ -340,9 +374,9 @@ void pn_status_init(void)
 	memset(g_ups_status, 0, sizeof(g_ups_status));
 }
 
-void pn_status_set(int type, long value)
+void pn_status_set(pn_info_t *pn_info_p, long value)
 {
-	g_ups_status[PN_STATUS_TYPE(type)] = value;
+	g_ups_status[PN_STATUS_INDEX(pn_info_p->flags)] = value;
 }
 
 void pn_status_commit(void)
@@ -350,7 +384,7 @@ void pn_status_commit(void)
 	char buf[PN_INFOSIZE];
 	pn_info_t *pn_info_p;
 
-	switch (g_ups_status[PN_STATUS_TYPE(PN_STATUS_PWR)]) {
+	switch (g_ups_status[PN_STATUS_INDEX(PN_STATUS_PWR)]) {
 	case PWR_OTHER:			/*  INFO_STATUS/? */
 	case PWR_REBOOTING:
 	case PWR_SOFT_BYPASS:
@@ -376,7 +410,7 @@ void pn_status_commit(void)
 		break;
 	}
 
-	switch (g_ups_status[PN_STATUS_TYPE(PN_STATUS_BATT)]) {
+	switch (g_ups_status[PN_STATUS_INDEX(PN_STATUS_BATT)]) {
 	case BATT_UNKNOWN:		/* unknown -> INFO_STATUS/? */
 	case BATT_NORMAL:		/* batteryNormal -> INFO_STATUS/? */
 		break;
@@ -386,14 +420,14 @@ void pn_status_commit(void)
 		break;
 	}
 
-	switch (g_ups_status[PN_STATUS_TYPE(PN_STATUS_CAL)]) {
+	switch (g_ups_status[PN_STATUS_INDEX(PN_STATUS_CAL)]) {
 	case CAL_INPROGRESS:
 		snprintfcat(buf, sizeof(buf),
 			(strlen(buf) == 0 ? "%s" : " %s"), "CAL");
 		break;
 	}
 
-	switch (g_ups_status[PN_STATUS_TYPE(PN_STATUS_RB)]) {
+	switch (g_ups_status[PN_STATUS_INDEX(PN_STATUS_RB)]) {
 	case RB_NEED:
 		snprintfcat(buf, sizeof(buf),
 			(strlen(buf) == 0 ? "%s" : " %s"), "RB");
@@ -451,14 +485,12 @@ struct instcmds_t *pn_find_instcmd(int cmd)
 /* walk ups variables and set elements of the info array. */
 void pn_ups_walk(int mode)
 {
-	static long iterations = 0;
+	static unsigned long iterations = 0;
 
 	pn_info_t *pn_info_p;
 	bool status;
 	struct netvars_t *netvar;
 	
-	upsdebugx(2, "in pn_ups_walk()");
-
 	pn_status_init();
 
 	for (pn_info_p = &pn_info[0]; pn_info_p->info_type; pn_info_p++) {
@@ -467,10 +499,6 @@ void pn_ups_walk(int mode)
 		if (pn_info_p->info_type & PN_CMD_MASK)
 			continue;
 
-		netvar = pn_find_netvar(pn_info_p->info_type);
-
-		upsdebugx(3, "pn_ups_walk: processing info %s", netvar->name);
-
 		/* skip elements we shouldn't show. */
 		if (!(pn_info_p->flags & PN_FLAG_OK))
 			continue;
@@ -505,14 +533,18 @@ void pn_ups_walk(int mode)
 
 		/* set stale flag if data is stale, clear if not. */
 		if (status == TRUE) {
-			if (pn_info_p->flags & PN_FLAG_STALE)
-				upslogx(LOG_INFO, "pn_ups_walk: data resumed for %s",
-					netvar->name);
+			if (pn_info_p->flags & PN_FLAG_STALE) {
+				netvar = pn_find_netvar(pn_info_p->info_type);
+				upslogx(LOG_INFO, "[%s] pn_ups_walk: data resumed for %s",
+					upsname, netvar->name);
+			}
 			pn_info_p->flags &= ~PN_FLAG_STALE;
 		} else {
-			if (!(pn_info_p->flags & PN_FLAG_STALE))
-				upslogx(LOG_INFO, "pn_ups_walk: data stale for %s",
-					netvar->name);
+			if (!(pn_info_p->flags & PN_FLAG_STALE)) {
+				netvar = pn_find_netvar(pn_info_p->info_type);
+				upslogx(LOG_INFO, "[%s] pn_ups_walk: data stale for %s",
+					upsname, netvar->name);
+			}
 			pn_info_p->flags |= PN_FLAG_STALE;
 		}
 
@@ -534,7 +566,7 @@ bool pn_ups_get(pn_info_t *pn_info_p)
 	case INFO_STATUS:
 		status = pn_snmp_get_int(pn_info_p->OID, &value);
 		if (status == TRUE)
-			pn_status_set(pn_info_p->flags, value);
+			pn_status_set(pn_info_p, value);
 
 		break;
 
@@ -553,13 +585,24 @@ bool pn_ups_get(pn_info_t *pn_info_p)
 	case INFO_RUNTIME:
 		status = pn_snmp_get_int(pn_info_p->OID, &value);
 		if (status == TRUE) {
-			/* convert milliseconds to minutes. */
-			snprintf(buf, sizeof(buf), "%ld", value / (60 * 100));
+			/* convert seconds to minutes. */
+			snprintf(buf, sizeof(buf), "%ld", value / 60);
 			pn_setinfo(pn_info_p->info_type, buf,
 				pn_info_p->info_flags, pn_info_p->info_len);
 		}
 		break;
 
+	case INFO_LINESENS:
+		status = pn_snmp_get_int(pn_info_p->OID, &value);
+		if (status == TRUE) {
+			if (value >= 1 && value <= 4) {
+				static char *linesens[] = { "A", "L", "M", "H" };
+				pn_setinfo(pn_info_p->info_type, linesens[value - 1],
+					pn_info_p->info_flags, pn_info_p->info_len);
+			}
+		}
+		break;
+
 	default:
 		/* get snmp OID value. */
 		status = pn_snmp_get_str(pn_info_p->OID,
@@ -582,42 +625,59 @@ void pn_ups_set(int type, int data_len, char *data)
 	bool ret;
 	struct netvars_t *netvar;
 
-	upsdebugx(2, "in pn_ups_set()");
-
 	pn_info_p = pn_find_info(type);
 	netvar = pn_find_netvar(type);
 
+	upslogx(LOG_INFO, "[%s] setting variable %s to %s",
+		upsname, netvar->name, data);
+
 	if (pn_info_p == NULL || pn_info_p->info_type == 0 ||
 		!(pn_info_p->flags & PN_FLAG_OK))
 	{
-		upslogx(LOG_ERR, "pn_ups_set: info element unavailable %s",
-			netvar->name);
+		upslogx(LOG_ERR, "[%s] pn_ups_set: info element unavailable %s",
+			upsname, netvar->name);
 		return;
 	}
 
 	if (! (pn_info_p->info_flags & FLAG_RW) || pn_info_p->OID == NULL) {
-		upslogx(LOG_ERR, "pn_ups_set: not writable %s", netvar->name);
+		upslogx(LOG_ERR, "[%s] pn_ups_set: not writable %s",
+			upsname, netvar->name);
 		return;
 	}
 
 	/* set value. */
 
-	if (PN_TYPE(pn_info_p->info_flags) == PN_TYPE_STRING) {
+	if (PN_TYPE(pn_info_p) == PN_TYPE_STRING) {
 		ret = pn_snmp_set_str(pn_info_p->OID, data);
+	} else if (PN_TYPE(pn_info_p) == PN_TYPE_TIME) {
+		ret = pn_snmp_set_time(pn_info_p->OID, strtol(data, NULL, 0));
 	} else {
-		ret = pn_snmp_set_int(pn_info_p->OID, strtol(data, NULL, 0));
+		switch (pn_info_p->info_type) {
+		case INFO_LINESENS: /* convert letter to int */
+			{
+				int i;
+				static char *linesens[] = { "A", "L", "M", "H" };
+
+				ret = FALSE;
+				for (i = 1; i <= 4; i++) {
+					if (!strcasecmp(linesens[i - 1],data)) {
+						ret = pn_snmp_set_int(pn_info_p->OID, i);
+						break;
+					}
+				}
+			}
+			break;
+		default:
+			ret = pn_snmp_set_int(pn_info_p->OID, strtol(data, NULL, 0));
+		}
 	}
 
 	if (ret == FALSE)
-		upslogx(LOG_ERR, "pn_ups_set: cannot set value %s for %s",
-			data, pn_info_p->OID);
-	else
-		upsdebugx(1, "pn_ups_set: sucessfully set %s to \"%s\"",
-			netvar->name, data);
+		upslogx(LOG_ERR, "[%s] pn_ups_set: cannot set value %s for %s",
+			upsname, data, pn_info_p->OID);
 
 	/* update info array. */
 	pn_setinfo(type, data, pn_info_p->info_flags, pn_info_p->info_len);
-	writeinfo();
 }
 
 /* process instant command and take action. */
@@ -627,16 +687,17 @@ void pn_ups_instcmd(int auxcmd, int data_len, char *data)
 	int status;
 	struct instcmds_t *instcmd;
 
-	upsdebugx(2, "in pn_ups_instcmd()");
-
 	pn_info_p = pn_find_info(auxcmd);
 	instcmd = pn_find_instcmd(auxcmd);
 
+	upslogx(LOG_INFO, "[%s] sending instant command %s",
+		upsname, instcmd->name);
+
 	if (pn_info_p->info_type == 0 || !(pn_info_p->flags & PN_FLAG_OK) ||
 		pn_info_p->OID == NULL)
 	{
-		upslogx(LOG_ERR, "pn_ups_instcmd: %s unavailable",
-			instcmd->name);
+		upslogx(LOG_ERR, "[%s] pn_ups_instcmd: %s unavailable",
+			upsname, instcmd->name);
 		return;
 	}
 
@@ -649,11 +710,8 @@ void pn_ups_instcmd(int auxcmd, int data_len, char *data)
 	}
 
 	if (status == FALSE)
-		upslogx(LOG_ERR, "pn_ups_instcmd: cannot set value for %s",
-			instcmd->name);
-	else
-		upsdebugx(1, "pn_ups_instcmd: successfully sent command %s",
-			instcmd->name);
+		upslogx(LOG_ERR, "[%s] pn_ups_instcmd: cannot set value for %s",
+			upsname, instcmd->name);
 }
 
 /* do the shutdown immediately. */
@@ -671,11 +729,9 @@ void pn_shutdown_ups(void)
 	/* logic from newapc.c */
 	switch (sdtype) {
 	case 3:		/* shutdown with grace period */
-		upslogx(LOG_INFO, "sending delayed power off command to UPS");
 		pn_ups_instcmd(CMD_SHUTDOWN, 0, 0);
 		break;
 	case 2:		/* instant shutdown */
-		upslogx(LOG_INFO, "sending power off command to UPS");
 		pn_ups_instcmd(CMD_OFF, 0, 0);
 		break;
 	case 1:
@@ -683,7 +739,8 @@ void pn_shutdown_ups(void)
 		/* if the UPS gets power during shutdown process */
 		/* Specifically it sends both the soft shutdown 'S' */
 		/* and the powerdown after grace period - '@000' commands */
-		upslogx(LOG_INFO, "UPS - sending shutdown/powerdown");
+		upslogx(LOG_INFO, "[%s] UPS - sending shutdown/powerdown",
+			upsname);
 		if (pwr_status == PWR_BATTERY)
 			pn_ups_instcmd(CMD_SOFTDOWN, 0, 0);
 		pn_ups_instcmd(CMD_SDRET, 0, 0);
@@ -692,11 +749,13 @@ void pn_shutdown_ups(void)
 		/* if on battery... */
 		if (pwr_status == PWR_BATTERY) {
 			upslogx(LOG_INFO,
-				"UPS is on battery, sending shutdown command...");
+				"[%s] UPS is on battery, sending shutdown command...",
+				upsname);
 			pn_ups_instcmd(CMD_SOFTDOWN, 0, 0);
 		} else {
 			upslogx(LOG_INFO,
-				"UPS is online, sending shutdown+return command...");
+				"[%s] UPS is online, sending shutdown+return command...",
+				upsname);
 			pn_ups_instcmd(CMD_SDRET, 0, 0);
 		}
 		break;
@@ -709,8 +768,6 @@ void pn_shutdown_ups(void)
 
 void upsdrv_initinfo(void)
 {
-	upsdebugx(2, "in upsdrv_initinfo()");
-
 	/* add instant commands to the info database. */
 	pn_init_instcmds();
 
@@ -723,20 +780,14 @@ void upsdrv_initinfo(void)
 
 void upsdrv_updateinfo(void)
 {
-	upsdebugx(2, "in upsdrv_updateinfo()");
-  
 	/* update all dynamic info fields */
 	pn_ups_walk(PN_WALKMODE_UPDATE);
-
-	writeinfo();
 }
 
 /* do the shutdown immediately and cleanup for exit. */
 /* upsdrv_initups already called, but upsdrv_initinfo isn't. */
 void upsdrv_shutdown(void)
 {
-	upsdebugx(2, "in upsdrv_shutdown");
-
 	pn_shutdown_ups();
 	pn_cleanup();
 }
@@ -755,8 +806,6 @@ void upsdrv_help(void)
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
-	upsdebugx(2, "in upsdrv_makevartable()");
-
 	addvar(VAR_VALUE, PN_VAR_SDTYPE,
 		"Specify shutdown type (1-3, default 0)");
 	addvar(VAR_VALUE, PN_VAR_COMMUNITY,
@@ -765,12 +814,8 @@ void upsdrv_makevartable(void)
 
 void upsdrv_banner(void)
 {
-	upslogx(1,"Network UPS Tools version %s", UPS_VERSION);
-	upslogx(1,"APC Powernet SNMP driver version %s", upsdrv_version);
-	upslogx(1,"Revision %s\n", upsdrv_cvsid);
-
-	/* main.c prints a warning. */
-	experimental_driver = 1;
+	upslogx(1,"Network UPS Tools %s APC Powernet SNMP driver %s",
+		UPS_VERSION, PN_UPSDRV_VERSION);
 }
 
 void upsdrv_initups(void)
@@ -779,8 +824,6 @@ void upsdrv_initups(void)
 	char model[PN_INFOSIZE];
 	bool status;
 
-	upsdebugx(2, "in upsdrv_initups()");
-
 	/* init SNMP library, etc... */
 	pn_startup();
 
@@ -794,8 +837,6 @@ void upsdrv_initups(void)
 			OID_POWERNET_MIB, g_snmp_sess.peername);   
 }
 
-/* tell main how many items you need */
-int upsdrv_infomax(void)
+void upsdrv_cleanup(void)
 {
-	return NUM_INFO_ITEMS;
 }
