@@ -934,7 +934,7 @@ void read_mibconf(char *mib)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, errmsg);
+			upslogx(LOG_WARNING, "%s", errmsg);
 		}
 	}
 	pconf_finish(&ctx);		
