@@ -43,13 +43,20 @@
 #include "proto.h"
 #include "version.h"
 
+/* get the syslog ready for us */
+void open_syslog(const char *progname);
+
 /* close ttys and become a daemon */
 void background(void);
 
-void switch_to_user(const char *user, const char *group);
+/* do this here to keep pwd/grp stuff out of the main files */
+struct passwd *get_user_pwent(const char *name);
 
-/* change uid/gid if running as root */
-void droproot(void);
+/* change to the user defined in the struct */
+void become_user(struct passwd *pw);
+
+/* drop down into a directory and throw away pointers to the old path */ 
+void chroot_start(const char *path);
 
 /* write a pid file - <name> is a full pathname *or* just the program name */
 void writepid(const char *name);
@@ -70,6 +77,15 @@ void xbit_clear(int *val, int flag);
 /* enable writing upslog() type messages to the syslog */
 void syslogbit_set(void);
 
+/* Return the default path for the directory containing configuration files */
+const char * confpath(void);
+
+/* Return the default path for the directory containing state files */
+const char * dflt_statepath(void);
+
+/* Return the alternate path for pid files */
+const char * altpidpath(void);
+
 void upslog(int priority, const char *fmt, ...);
 void upslogx(int priority, const char *fmt, ...);
 void upsdebug(int level, const char *fmt, ...);
@@ -90,10 +106,8 @@ char *xstrdup(const char *string);
 #define SMALLBUF	512
 #define LARGEBUF	1024
 
-/* 
- * Provide declarations for getopt() global variables, used in the model
- * drivers for parsing command line options.
- */
+/* Provide declarations for getopt() global variables */
+
 #ifdef NEED_GETOPT_H
 #include <getopt.h>
 #else
