@@ -1,24 +1,14 @@
 #ifndef NUT_SHARED_H
 #define NUT_SHARED_H
 
-/* shared.h - shared data structures for upsd and model-specific modules */
+/* shared.h - legacy variable naming and mapping */
 
-/* itype - holds info (variable name, variable value) */
+/* OLD STUFF - do not add anything to this list! */
 
-/* size of the values stored within the array */
-#define VALSIZE 32
+/* ------------------------------------------------------- *
+ * Included for compatibility in 1.4, will be gone by 2.0  *
+ * ------------------------------------------------------- */
 
-/* used in an array and stored in shared memory or to state file */
-typedef struct {
-	int	type;
-	char	value[VALSIZE];
-	int	flags;			/* see FLAGS_* below 		*/
-	int	auxdata;		/* length, # entries, enumtype	*/
-}	itype;
-
-/* these values are only used between processes on the same host... */
-
-#define INFO_MEMBERS	0x0000		/* number of info entries 	*/
 #define INFO_MFR	0x0001		/* manufacturer string		*/
 #define INFO_MODEL	0x0002		/* model string			*/
 #define INFO_SERIAL	0x0003		/* serial number		*/
@@ -182,44 +172,11 @@ typedef struct {
 #define INFO_HIBOOSTXFER        0x0094  /* Upper threshold for boosting */
 #define INFO_BATT_RUNTIME       0x00a0  /* Remaining time on batt [secs]*/
 
-#define	INFO_SYSMASK	0xF000		/* mask for system level INFOs  */
-#define	INFO_MTIME	0xFFFA		/* last modified time		*/
+/* what's left of the internal special types */
 #define INFO_INSTCMD	0xFFFB		/* supported instant command	*/
-#define INFO_ENUM	0xFFFC		/* holder for an ENUM value	*/
-#define	INFO_MSGID	0xFFFD		/* message queue ID             */
-#define INFO_SHMID	0xFFFE		/* shared memory struct ID      */
 #define INFO_UNUSED	0xFFFF		/* this entry is not being used */
 
-/* message queue identifiers */
-
-#define UPSMSG_TOUPSD	0x1000		/* message to upsd from model   */
-#define UPSMSG_TOMODEL	0x1001		/* message from upsd to model   */
-#define UPSMSG_MAXLEN	128		/* maximum length permitted	*/
-
-/* message queue struct commands */
-
-#define UPSMSG_CMDMASK	0xA100		/* command mask			*/
-#define UPSMSG_CMDSDD	0xA101		/* dead, replaced by instcmd    */
-#define UPSMSG_CMDSET	0xA102		/* set variable to value	*/
-#define UPSMSG_INSTCMD	0xA103		/* do an instant command	*/
-
-#define UPSMSG_REPMASK	0xAE00		/* reply mask			*/
-#define UPSMSG_REPOK	0xAE01		/* command completed OK		*/
-#define UPSMSG_REPAGAIN 0xAE02		/* try command again		*/
-
-#define UPSMSG_ERRMASK	0xAF00		/* error mask			*/
-#define UPSMSG_ERRNI	0xAF01		/* error: cmd not implemented	*/
-#define UPSMSG_ERRRO	0xAF02		/* error: read-only variable	*/
-#define UPSMSG_ERRCF	0xAF03		/* error: command failed	*/
-
-typedef struct {
-	int	cmdtype;
-	int	auxcmd;
-	int	dlen;
-	char	data[UPSMSG_MAXLEN];		/* up to dlen bytes long */
-}	msgtype;
-
-/* flags from the info array */
+/* old flags from the info array */
 #define FLAG_RW		0x0001		/* variable is read/write	  */
 #define FLAG_ENUM	0x0002		/* variable is an enumerated type */
 #define FLAG_STRING	0x0004		/* variable is a string           */
@@ -234,15 +191,13 @@ typedef struct {
 #define CMD_CAL0	0x2005		/* stop runtime calibration	  */
 #define CMD_CAL1	0x2006		/* start runtime calibration	  */
 #define CMD_FPTEST	0x2007		/* test the front panel		  */
-#define CMD_DUMB	0x2008		/* Turn dumb			  */
+
 #define CMD_SIMPWF	0x2009		/* Simulate power failure	  */
 #define CMD_SOFTDOWN	0x200A		/* Soft shutdown		  */
 #define CMD_SHUTDOWN	0x200B		/* Shutdown after grace period	  */
 #define CMD_BYPASS	0x200C		/* Switch to bypass mode	  */
 #define CMD_RSTMINMAX	0x200D	/* Reset min and max utility readings */
-
-/* info access functions */
-
-int findinfo(const itype *info, int size, int offset, int type);
+#define CMD_STOPSHUTD   0x200E  /* Stop Shutdown                      */
+#define CMD_WATCHDOG    0x200F  /* Shutdown in 5m restart in 3m, for crontab Watchdog */
 
 #endif /* NUT_SHARED_H */
