@@ -17,6 +17,14 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#ifndef CTYPE_H_SEEN
+#define CTYPE_H_SEEN 1
+
+#ifdef HAVE_SSL
+#include <openssl/err.h>
+#include <openssl/ssl.h>
+#endif
+
 /* client structure */
 typedef struct {
 	char	*addr;
@@ -37,3 +45,5 @@ typedef struct {
 
 	void	*next;
 }	ctype;
+
+#endif	/* CTYPE_H_SEEN */
