@@ -17,16 +17,20 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
-#include "upsd.h"
-#include "user.h"
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
+#include "common.h"
 #include "parseconf.h"
 
-	extern	instcmds_t instcmds[];
-	extern	acltype	*firstacl;
+#include "user.h"
+#include "user-data.h"
+#include "access.h"
+
 	ulist_t	*users = NULL;
 
-	static	int	oldfilewarn = 0;
 	static	ulist_t	*curr_user;
 
 /* create a new user entry */
@@ -81,7 +85,7 @@ static acllist *addallow(acllist *base, const char *acl)
 		tmp = tmp->next;
 	}
 
-	tmp = xmalloc(sizeof(acltype));
+	tmp = xmalloc(sizeof(struct acl_t));
 	tmp->aclname = xstrdup(acl);
 	tmp->next = NULL;
 
@@ -126,52 +130,11 @@ static void user_password(const char *pw)
 	curr_user->password = xstrdup(pw);
 }
 
-static instcmdlist *addinstcmd(instcmdlist *base, const char *cmd)
-{
-	instcmdlist	*tmp, *last;
-	int	cmdnum = -1, i;
-
-	if (!cmd)
-		return base;
-
-	tmp = last = base;
-
-	while (tmp) {
-		last = tmp;
-		tmp = tmp->next;
-	}
-
-	/* 'all' is a special case */
-	if (!strcasecmp(cmd, "all"))
-		cmdnum = CMD_ALL;
-	else {
-		for (i = 0; instcmds[i].name != NULL; i++)
-			if (!strcasecmp(instcmds[i].name, cmd)) {
-				cmdnum = instcmds[i].cmd;
-				break;
-			}		
-	}
-
-	if (cmdnum == -1) {
-		fprintf(stderr, "Ignoring unknown command name: %s\n", cmd);
-		return base;
-	}
-
-	tmp = xmalloc(sizeof(instcmdlist));
-	tmp->cmd = cmdnum;
-	tmp->next = NULL;
-
-	if (last) {
-		last->next = tmp;
-		return base;
-	}
-
-	return tmp;
-}
-
 /* attach allowed instcmds to user */
 static void user_add_instcmd(const char *cmd)
 {
+	instcmdlist	*tmp, *last;
+
 	if (!curr_user) {
 		upslogx(LOG_WARNING, "Ignoring instcmd definition outside "
 			"user section");
@@ -181,7 +144,28 @@ static void user_add_instcmd(const char *cmd)
 	if (!cmd)
 		return;
 
-	curr_user->firstcmd = addinstcmd(curr_user->firstcmd, cmd);
+	tmp = curr_user->firstcmd;
+	last = NULL;
+
+	while (tmp) {
+		last = tmp;
+
+		/* ignore duplicates */
+		if (!strcasecmp(tmp->cmd, cmd))
+			return;
+
+		tmp = tmp->next;
+	}
+
+	tmp = xmalloc(sizeof(instcmdlist));
+
+	tmp->cmd = xstrdup(cmd);
+	tmp->next = NULL;
+
+	if (last)
+		last->next = tmp;
+	else
+		curr_user->firstcmd = tmp;
 }
 
 static actionlist *addaction(actionlist *base, const char *action)
@@ -248,6 +232,8 @@ static void flushcmd(instcmdlist *first)
 	while (ptr) {
 		next = ptr->next;
 
+		if (ptr->cmd)
+			free(ptr->cmd);
 		free(ptr);
 
 		ptr = next;
@@ -290,10 +276,10 @@ void user_flush(void)
 		if (ptr->password)
 			free(ptr->password);
 
-		if (ptr->firstcmd);
+		if (ptr->firstcmd)
 			flushcmd(ptr->firstcmd);
 
-		if (ptr->firstaction);
+		if (ptr->firstaction)
 			flushaction(ptr->firstaction);
 
 		free(ptr);
@@ -304,16 +290,18 @@ void user_flush(void)
 	users = NULL;
 }	
 
-int user_matchacl(ulist_t *user, struct sockaddr_in *addr)
+static int user_matchacl(ulist_t *user, const struct sockaddr_in *addr)
 {
-	acllist	*tmp = user->firstacl;
+	acllist	*tmp;
+
+	tmp = user->firstacl;
 
 	/* no acls means no access (fail-safe) */
 	if (!tmp)
 		return 0;		/* good */
 
 	while (tmp) {
-		if (checkacl(tmp->aclname, addr) == 1)
+		if (acl_check(tmp->aclname, addr) == 1)
 			return 1;	/* good */
 	
 		tmp = tmp->next;
@@ -322,7 +310,7 @@ int user_matchacl(ulist_t *user, struct sockaddr_in *addr)
 	return 0;	/* fail */
 }
 
-int user_matchinstcmd(ulist_t *user, int cmd)
+static int user_matchinstcmd(ulist_t *user, const char * cmd)
 {
 	instcmdlist	*tmp = user->firstcmd;
 
@@ -331,7 +319,8 @@ int user_matchinstcmd(ulist_t *user, int cmd)
 		return 0;	/* fail */
 
 	while (tmp) {
-		if ((tmp->cmd == cmd) || (tmp->cmd == CMD_ALL))
+		if ((!strcasecmp(tmp->cmd, cmd)) || 
+			(!strcasecmp(tmp->cmd, "all")))
 			return 1;	/* good */
 		tmp = tmp->next;
 	}
@@ -339,23 +328,25 @@ int user_matchinstcmd(ulist_t *user, int cmd)
 	return 0;	/* fail */
 }
 
-int user_checkinstcmd(struct sockaddr_in *addr, char *username, char *password,
-               int cmd)
+int user_checkinstcmd(const struct sockaddr_in *addr, 
+	const char *un, const char *pw, const char *cmd)
 {
 	ulist_t	*tmp = users;
 
+	if ((!un) || (!pw) || (!cmd))
+		return 0;	/* failed */
+
 	while (tmp) {
 
 		/* let's be paranoid before we call strcmp */
 
-		if ((!tmp->username) || (!tmp->password) ||
-		   (!username) || (!password)) {
+		if ((!tmp->username) || (!tmp->password)) {
 			tmp = tmp->next;
 			continue;
 		}
 
-		if (!strcmp(tmp->username, username)) {
-			if (!strcmp(tmp->password, password)) {
+		if (!strcmp(tmp->username, un)) {
+			if (!strcmp(tmp->password, pw)) {
 				if (!user_matchacl(tmp, addr))
 					return 0;		/* fail */
 
@@ -377,7 +368,7 @@ int user_checkinstcmd(struct sockaddr_in *addr, char *username, char *password,
 	return 0;	/* fail */
 }
 
-int user_matchaction(ulist_t *user, char *action)
+static int user_matchaction(ulist_t *user, const char *action)
 {
 	actionlist	*tmp = user->firstaction;
 
@@ -394,23 +385,25 @@ int user_matchaction(ulist_t *user, char *action)
 	return 0;	/* fail */
 }
 
-int user_checkaction(struct sockaddr_in *addr, char *username, char *password,
-                     char *action)
+int user_checkaction(const struct sockaddr_in *addr, 
+	const char *un, const char *pw, const char *action)
 {
 	ulist_t	*tmp = users;
 
+	if ((!un) || (!pw) || (!action))
+		return 0;	/* failed */
+
 	while (tmp) {
 
 		/* let's be paranoid before we call strcmp */
 
-		if ((!tmp->username) || (!tmp->password) ||
-		   (!username) || (!password)) {
+		if ((!tmp->username) || (!tmp->password)) {
 			tmp = tmp->next;
 			continue;
 		}
 
-		if (!strcmp(tmp->username, username)) {
-			if (!strcmp(tmp->password, password)) {
+		if (!strcmp(tmp->username, un)) {
+			if (!strcmp(tmp->password, pw)) {
 
 				if (!user_matchacl(tmp, addr)) {
 					upsdebugx(2, "user_matchacl: failed");
@@ -508,26 +501,17 @@ static void parse_rest(char *var, char *fval, char **arg, int next, int left)
 		parse_var(var, arg[next + i]);
 }
 
-void user_parseconf(int numargs, char **arg)
+static void user_parse_arg(int numargs, char **arg)
 {
 	char	*ep;
 
 	if ((numargs == 0) || (!arg))
 		return;
 
-	/* TODO: remove this after awhile */
-
-	/* don't attempt to parse any old files */
-	if (oldfilewarn == 1)
+	/* ignore old file format */
+	if (!strcasecmp(arg[0], "user"))
 		return;
 
-	/* detect old file format */
-	if ((!strcasecmp(arg[0], "user")) && (oldfilewarn == 0)) {
-		upslogx(LOG_ERR, "Unable to parse old style upsd.users\n");
-		oldfilewarn = 1;
-		return;
-	}
-
 	/* handle 'foo=bar' (compressed form) */
 
 	ep = strchr(arg[0], '=');
@@ -577,24 +561,41 @@ void user_parseconf(int numargs, char **arg)
 	/* ... unhandled ... */
 }
 
-void user_parseerror(int linenum, char *errtext)
+/* called for fatal errors in parseconf like malloc failures */
+void upsd_user_err(const char *errmsg)
 {
-	if (linenum != 0)
-		upslogx(LOG_ERR, "%s/upsd.users line %d: %s",
-			CONFPATH, linenum, errtext);
-	else
-		upslogx(LOG_ERR, "%s/upsd.users: %s",
-			CONFPATH, errtext);
+        upslogx(LOG_ERR, "Fatal error in parseconf(upsd.users): %s", errmsg);
 }
 
 void user_load(void)
 {
-	char	ufn[SMALLBUF];
+	char	fn[SMALLBUF];
+	PCONF_CTX	ctx;
 
 	curr_user = NULL;
 
-	snprintf(ufn, sizeof(ufn), "%s/upsd.users", CONFPATH);
+	snprintf(fn, sizeof(fn), "%s/upsd.users", confpath());
 
-	oldfilewarn = 0;
-	parseconf(ufn, user_parseconf, user_parseerror);
+	check_perms(fn);
+
+	pconf_init(&ctx, upsd_user_err);
+
+	if (!pconf_file_begin(&ctx, fn)) {
+		pconf_finish(&ctx);
+
+		upslogx(LOG_WARNING, "%s", ctx.errmsg);
+		return;
+	}
+
+	while (pconf_file_next(&ctx)) {
+		if (pconf_parse_error(&ctx)) {
+			upslogx(LOG_ERR, "Parse error: %s:%d: %s",
+				fn, ctx.linenum, ctx.errmsg);
+			continue;
+		}
+
+		user_parse_arg(ctx.numargs, ctx.arglist);
+	}
+
+	pconf_finish(&ctx);
 }
